
package vt.cs.db;

import java.util.*;

/**
 * A Database is a set of tables (Table objects). Currently the
 * only query method present is the retrieval of a Table object ref
 * by name. Other methods for querying by type, etc... will need to be
 * added. The current prototype implementation also does not address
 * versioning.
 */
public abstract class Database extends AttributeObject {
	public static final String PREFIX = "Database";

	protected	Hashtable	tables	= null;

	public Database() {
		tables = new Hashtable();
	}

	/**
	 * Returns array of all available Tables 
	 * <I>Return a Vector or Enumeration instead?</I>
	 */
	public String[]	getTables() {
	 	String[]    tbs = new String[tables.size()];
        int         i   = 0;
        for (Enumeration e = tables.keys(); e.hasMoreElements();) {
            tbs[i++] = (String)e.nextElement();
        }
        return tbs;
	}

	/**
	 * Creates a new Table in the database. Probably need to come up with
	 * a better name for the 'descriptions' parm -- this is meant to 
	 * contain information such as the units of measure in which values
	 * are expressed and other similar data which might be useful for
	 * presentation.
	 */
	public abstract Table newTable(String name, String[] columnNames, 
		String[] columnTypes, String[] columnDescriptions, String source);

	/**
	 * Gets the table with the given name
	 */
	public Table getTable(String name) {
		return (Table) tables.get(name);
	}

	/**
	 * Deletes a table. As in the DatabaseSet, we need a way to
	 * handle failures in read-only environments.
	 */
	public void deleteTable(String name) {
		tables.remove(name);
	}

	/**
	 * Removes all Tables from the database. 
	 */
	public void clear() {
		tables.clear();
	}

	/**
	 * Not sure about this one. If it does belong in the interface,
	 * it should possibly be renamed to something like cloneTable. (??)
	 */
	public abstract void addTable(Table table);
	
}

