package vt.cs.db.filedb;

import java.util.*;
import vt.cs.db.*;

/**
 * DatabaseSet Implementation for "filedb" package.
 * Assumes a single Database containing a single table.<p>
 * TODO: Fix exception handling--make it consistent throughout
 * @author Jeff Nielsen
 */
public class DatabaseSetImpl extends vt.cs.db.DatabaseSet {
    private FileDatabase db = null;

    /**
     * Acts as the constructor for a DatabaseSetImpl Object.
     * The actual constructor called cannot take any parameters,
     * but calls init().
     * @param url The url location of the file containing the table
     */
    public void init(String url) throws DatabaseException {
        // setName(url);
        db = new FileDatabase(url);
    }

    /**
     * Returns the name of the single Database object in this set.
     * The name will be the same as the DatabaseSetImpl name.
     */
    public String[] getDatabases() {
        String[] dbs = new String[1];
        dbs[0] = db.getName();
        return dbs;
    }

    /**
     * Always returns the single database in this set, regardless
     * of the parameter passed to it.
     */
    public Database getDatabase(String name) { return db; }

    /**
     * Currently does nothing
     */
    public Database newDatabase(String name) { return null; }

    /**
     * Currently does nothing
     */
    public void addDatabase(Database db) {}

    /**
     * Currently does nothing
     */
    public void deleteDatabase(String name) {}

    }

class FileDatabase extends Database {

    /**
     * Again, assume only one table.
     */
    public FileDatabase(String url) throws DatabaseException {
        // setName(url);
        DiskFileTable table = new DiskFileTable(url);
        tables.put("mytable", table);
    }

    /**
     * Override base class implementation, since we only have one table and
     * don't care about the parameter..
     */
    public Table getTable(String name) { return (Table) tables.get("mytable"); }

    public void deleteTable(String name) {}
    public void addTable(Table table) {}
    public void clear() {}
    public Table newTable(String name, String[] columnNames,
        String[] columnTypes, String[] columnDescriptions, String source) { 
        return null; 
    }
}


/**
 * Helper class--a Variable object stores all the information we 
 * need about a variable
 */
class Variable {
    public String name  = null; // Required
    public String description = null;   // Optional
    public Class type = null;  // Required
    public Hashtable valLabels = null;
    public Hashtable attributes = null;

    Variable(String name, Class type, String description) {
        this.name = name;
        this.description = description;
        this.type = type;
    }

    Variable(String name, Class type) { this(name, type, null); }

    public void addValLabel(int code, String meaning) {
        if (valLabels == null) valLabels = new Hashtable();
        valLabels.put(new Integer(code), meaning);
    }
    public void addAttr(String name, String value) {
        if (attributes == null) attributes = new Hashtable();
        attributes.put(name, value);
    }
}

class Tag {
    public  String name;
    public Hashtable attlist;

    Tag(String name) { this.name = name; attlist = new Hashtable(); }
    String attribute(String attname) {
        return (String)attlist.get(attname);
    }
}

class CodebookSyntaxException extends DatabaseException {
        CodebookSyntaxException(int linenum) { 
            super("Bad Syntax in Codebook at line " + linenum); 
        }
}

class MissingVal {
    public String toString() { return "Missing_Value"; }
}

