
package vt.cs.db.rmidb;

import java.rmi.*;
import java.util.*;
import vt.cs.db.Table;

public interface RemoteDatabase extends Remote {
	public static final String PREFIX = "Database";

	public void	setName(String name) throws RemoteException;

	public String getName() throws RemoteException;

    /**
     * Sets the attribute with name 'key' to the specified value.
     */
    public void setAttribute(String key, Object value) throws RemoteException;
 
    /**
     * Removes the specified attribute.
     */
    public String removeAttribute(String key) throws RemoteException;
 
    /**
     * Gets the specified attribute.
     */
    public Object getAttribute(String key) throws RemoteException;
 
    /**
     * Gets an enumeration of all of this RemoteTable's attributes.
     */
    public Vector attributeKeys() throws RemoteException;

	/**
	 * Returns array of all available Tables 
	 */
	public String[]	getTables() throws RemoteException;

	/**
	 * Creates a new RemoteTable in the database. Probably need to come up with
	 * a better name for the 'descriptions' parm -- this is meant to 
	 * contain information such as the units of measure in which values
	 * are expressed and other similar data which might be useful for
	 * presentation.
	 */
	public RemoteTable newTable(String name, String[] columnNames, 
		String[] columnTypes, String[] columnDescriptions, String source)
		throws RemoteException;

	/**
	 * Gets the table with the given name
	 */
	public RemoteTable getTable(String name) throws RemoteException;

	/**
	 * Deletes a table. As in the DatabaseSet, we need a way to
	 * handle failures in read-only environments.
	 */
	public void deleteTable(String name) throws RemoteException;

	/**
	 * Removes all Tables from the database. 
	 */
	public void clear() throws RemoteException;

	/**
	 * Not sure about this one. If it does belong in the interface,
	 * it should possibly be renamed to something like cloneTable. (??)
	 */
	public void addTable(RemoteTable table) throws RemoteException;
	
}

