
package vt.cs.db.rmidb;

import java.util.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import vt.cs.db.*;

public class RemoteDatabaseImpl extends UnicastRemoteObject implements 
	RemoteDatabase {

	private Hashtable 	tables = null;
	private String		name = null;
	private Hashtable   attributes = null;

	RemoteDatabaseImpl(String name) throws RemoteException {
		super();
		this.name = name;
		tables = new Hashtable();
		attributes = new Hashtable();
	}

	RemoteDatabaseImpl() throws RemoteException {
		this("(Unnamed)");
	}

	public void setName(String name) throws RemoteException {
		this.name = name;
	}

	public String getName() throws RemoteException {
		return name;
	}

    public void setAttribute(String key, Object value) throws RemoteException {
        attributes.put(key, value);
    }
    public String removeAttribute(String key) throws RemoteException {
        return (String)attributes.remove(key);
    }
    public Object getAttribute(String key) throws RemoteException {
        return attributes.get(key);
    }
    public Vector attributeKeys() throws RemoteException {
        Vector      keys    = new Vector();
 
        for(Enumeration e = attributes.keys(); e.hasMoreElements();) {
            keys.addElement(e.nextElement());
        }
        return keys;
    }

	public RemoteTable newTable(String name, String[] columnNames, 
        String[] columnTypes, String[] columnDescriptions, String source) 
		throws RemoteException {

		RemoteTableImpl table = new RemoteTableImpl(name, columnNames, 
			columnTypes, columnDescriptions, source);

		addTable((RemoteTable)table);

		return (RemoteTable) table;
	}

	public void addTable(RemoteTable table) throws RemoteException {

		String objName = new String( Table.PREFIX + "/" + 
			this.name + "/" + table.getName());

		/* Add this to our dictionary and bind it to a URL */
		try {
			Naming.rebind(objName, (Remote)table);
			tables.put(table.getName(), "rmi://" +
				InetAddress.getLocalHost().getHostName() + "/" +
				objName);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
	}

	public void deleteTable(String name) throws RemoteException {
		/* Remove this from our dictionary and un-bind it from it's URL */
	}

	public String[] getTables() {
		String[] 	tabs = new String[tables.size()];
		int 		i = 0;

		for (Enumeration e = tables.keys(); e.hasMoreElements(); )
			tabs[i++] = (String)(e.nextElement());

		return tabs;
	}

	public RemoteTable getTable(String name) throws RemoteException {
		RemoteTable	table	= null;

		String objName = (String)(tables.get(name));
		if (objName != null) {
			try {
				table = (RemoteTable)(Naming.lookup(objName));
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
		return table;
	}

	/* See the "Unreferenced" interface... */
	public void clear() throws RemoteException {
		try {
			for (Enumeration e = tables.elements(); e.hasMoreElements(); ) {
				String objName = (String)(e.nextElement());
				Naming.unbind(objName);
			}
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}

