
package vt.cs.db.rmidb;

import java.rmi.*;
import java.util.*;

/**
 * A Remote Table of (currently) aribitrary objects. Only
 * primitive means for adding and extracting data are currently provided. 
 */
public interface RemoteTable extends Remote {
	/**
	 * Gets the name of the RemoteTable.
	 */
	public String getName() throws RemoteException;

	/**
	 * Puts the name of the RemoteTable.
	 */
	public void setName(String name) throws RemoteException;

	/**
	 * Sets the attribute with name 'key' to the specified value.
	 */
	public void setAttribute(String key, Object value) throws RemoteException;

	/**
	 * Removes the specified attribute.
	 */
	public String removeAttribute(String key) throws RemoteException;

	/**
	 * Gets the specified attribute.
	 */
	public Object getAttribute(String key) throws RemoteException;

	/**
	 * Gets an enumeration of all of this RemoteTable's attributes.
	 */
	public Vector attributeKeys() throws RemoteException;

	/**
	 * Returns the number of rows in the table
	 */
	public long rowcount() throws RemoteException;

	/**
	 * Returns a vector (or array?) of column names.
	 */
	public Vector columnNames() throws RemoteException;

	/**
	 * Returns a vector (or array?) of names of columns with data
	 * of the specified type. The type must be fully qualified, eg,
	 * "java.lang.Long".
	 */
	public Vector columnNames(String type) throws RemoteException;

	/**
	 * Returns the description string associated with the specified column.
	 */
	public String description(String columnName) throws RemoteException;

	/**
	 * Gets the data object at the specified row and column.
	 */
	public Object get(int rowNumber, int columnNumber) throws RemoteException;

	/**
	 * Gets the data object at the specified row and column.
	 */
	public Object get(int rowNumber, String columnName) throws RemoteException;

	/**
	 * Temporary hack...
     */
    public Vector getColumn(String columnName) throws RemoteException;
	/**
	 * Temporary hack...
     */
    public Vector getColumn(int columnNumber) throws RemoteException;

	/**
	 * Adds a row to the table. Currently no checking for size or
	 * type consistancy is done.
	 */
	public void appendRow(Object[] rowData) throws RemoteException;
}

