
/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/util/BoundedBufferVST.java,v 1.2 1997/05/16 22:31:15 begolej Exp $
 *
 */

package vt.cs.util;

/**
 *
 * @author	James "Bo" Begole
 *
 * @version $Revision $
 */


public class BoundedBufferVST implements BoundedBuffer {
  protected Object[]  elements; // the elements in the buffer
  protected int putIndex = 0;   // circular indices
  protected int takeIndex = 0;
  protected int usedSlots = 0;  // the count

  public BoundedBufferVST(int capacity) throws IllegalArgumentException {
    if (capacity <= 0)
      throw new IllegalArgumentException();
    elements = new Object[capacity];
  }

  public int getCount() {
    return usedSlots;
  }

  public int getCapacity() {
    return elements.length;
  }

  public synchronized void put(Object obj) {
    while (usedSlots == elements.length) // what until not full
      try { 
	wait();
      } catch (InterruptedException ex) {
	ex.printStackTrace();
      }

      elements[putIndex] = obj;
      putIndex = (putIndex +1) % elements.length;

      if (usedSlots++ == 0)  // signal if was empty
	notifyAll();
  }

  public synchronized Object take() {
    while (usedSlots == 0) // wait until not empty
      try {
	wait();
      } catch (InterruptedException ex) {
	ex.printStackTrace();
      }

      Object element = elements[takeIndex];
      elements[takeIndex] = null;
      takeIndex = (takeIndex +1) % elements.length;

      if (usedSlots-- == elements.length) // signal if was full
	notifyAll();

      return element;
  }
}
