/*
 *
 * Written By:   Philip Isenhour
 * Date:         25 September 1996
 * Description:  Prototype SharedData-based LabRoom replacement.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomApplet.java,v 1.5 1997/02/19 19:43:35 begolej Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.client;

import java.net.*;
import netscape.application.*;
import netscape.util.*;
import SharedData.*;
import vt.cs.virtSchool.server.*;

/**
 * The <CODE>RoomApplet</CODE> is responsible for presenting a list of
 * available rooms to the user, and instantiating a <CODE>RoomView</CODE>
 * when the user selects one.
 *
 * @version $Id: RoomApplet.java,v 1.5 1997/02/19 19:43:35 begolej Exp $
 */
public class RoomApplet extends Application implements Client {
  private String hostname = null;
  private int hostport = 0;
  private String name = null;
  private RoomList roomList = null;
  private UserList userList = null;
  private LoginView loginView = null;

  public void init() {
    super.init();

    if (parameterNamed("server") != null) {
      hostname = parameterNamed("server");
    }
    else {
      try {
	hostname = InetAddress.getLocalHost().getHostName();
      }
      catch(Exception e) {
	e.printStackTrace();
      }
    }

    if (parameterNamed("port") != null) {
      hostport = Integer.parseInt(parameterNamed("port"));
    }
    else {
      hostport = WorldServer.PORT;
    }
 
    if (parameterNamed("name") != null) {
      name = parameterNamed("name");
      login(name);
    }

    if (name == null) {
      loginView = new LoginView((mainRootView().width() - 200)/2, 
				(mainRootView().height()-100)/2, 200, 100, this);
      mainRootView().addSubview(loginView);
      loginView.textField.setFocusedView();
    }
    Environment.init(AWTCompatibility.awtApplet());
  }

  public void login(String name) {
    this.name = name;

    userList = new UserList(320, 10, 300, 200);
    roomList = new RoomList(10, 10, 300, 200, name, hostname, userList);

    ScrollGroup userScroll = new ScrollGroup(320, 10, 300, 200);
    ScrollGroup roomScroll = new ScrollGroup(10, 10, 300, 200);
    userScroll.scrollView().setContentView(userList);
    userScroll.setHasVertScrollBar(true);
    userScroll.setHasHorizScrollBar(true);
    userScroll.setBorder(new LineBorder(Color.black));
    roomScroll.setContentView(roomList);
    roomScroll.setHasVertScrollBar(true);
    roomScroll.setHasHorizScrollBar(true);
    roomScroll.setBorder(new LineBorder(Color.black));
    mainRootView().addSubview(userScroll);
    mainRootView().addSubview(roomScroll);
    mainRootView().setColor(Color.lightGray);
    mainRootView().draw();
  }

  public void stop() {
  }

  /* ----------- Client Interface methods ----------- */
  public String authenticate(String challenge) {
    System.out.println("RoomApplet: athenticate: challenge="+challenge);
    return null;
  }
  public String getName() {
    return name;
  }
  public void clientChanged(ClientEvent event) {
        System.out.println("RoomApplet: clientChanged. event="+event);
  }
  /* ------------------------------------------------ */

  public static void main(String args[]) {
    RoomApplet 		app			= null;
    ExternalWindow 	mainWindow 	= null;
    Size			size 		= null;

    System.setSecurityManager(new java.rmi.RMISecurityManager());

    app = new RoomApplet();
    mainWindow = new ExternalWindow();
    size = mainWindow.windowSizeForContentSize(630, 300);
    mainWindow.sizeTo(size.width, size.height);
    mainWindow.show();
    app.setMainRootView(mainWindow.rootView());
    app.run();
    app.loginView.setFocusedView();
  }

}


class RoomList extends ListView implements Target, WindowOwner {
  private	UserList			userList	= null;
  private	String				hostname	= null;
  private RoomData[] 			rooms		= null;
  private Session				session		= null;
  private String				clientName	= null;
  private	java.util.Hashtable	windows		= null;
  private int					lastIndex	= -1;

  public RoomList(int x, int y, int w, int h, String clientName,
		  String hostname, UserList userList) {
    super(x, y, w, h);

    this.hostname = hostname;
    this.userList = userList;
    this.clientName = clientName;
    this.windows = new java.util.Hashtable();

    setPrototypeItem(new ListItem());
    setCommand("click");
    setDoubleCommand("doubleclick");
    setTarget(this);
    setAllowsEmptySelection(false);
    setAllowsMultipleSelection(false);
    setTransparent(false);
    setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);
    setVertResizeInstruction(View.HEIGHT_CAN_CHANGE);

    update();
    performCommand("click", this);
  }

  void update() {
    int			i		= 0;

    removeAllItems();
    rooms = WorldServer.getRooms(hostname);
    try {
      for (i = 0; i < rooms.length; i++) {
	addItem(rooms[i].getRoomName());
      }
      if (session != null)
	userList.update(session);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    draw();
  }

  public void addItem(String name) {
    ListItem	item	= addItem();
    item.setTitle(name);
  }

  public void performCommand(String command, Object object) {
    if (command.equals("click")) {
      if (selectedIndex() != lastIndex) {
	try {
	  session = SessionFactory.createSession(
                rooms[selectedIndex()].getSessionURL());
	  userList.update(session);
	}
	catch (Exception e) {
	  e.printStackTrace();
	}
	lastIndex = selectedIndex();
      }
    }
    else if (command.equals("doubleclick")) {
      ListItem 		item = selectedItem();
      ExternalWindow	win  = (ExternalWindow) windows.get(item.title());
      RoomView		roomView	= null;
      Size			size 		= null;

      if (win != null) {
	win.show();
      }
      else {
	//win = createRoomWindow(rooms[selectedIndex()]);
	win = new ExternalWindow();
	size = win.windowSizeForContentSize(600, 600);
	win.sizeTo(size.width, size.height);
	roomView = new RoomView(size.width, size.height, 
				rooms[selectedIndex()], clientName);
	roomView.setOtherOwner(this);
	win.setOwner(roomView);
	win.rootView().addSubview(roomView);
	update();
	win.setTitle(item.title());
	windows.put(item.title(), win);
	win.show();
      }
      selectItem(item);
      draw();
    }
  }

  /* ---------- WindowOwner Interface Methods --------- */
  public void windowDidBecomeMain(Window win){}
  public void windowDidHide(Window win){}
  public void windowDidResignMain(Window win){}
  public void windowDidShow(Window win){}
  public boolean windowWillHide(Window win){
    windows.remove(win.title());
    return true;
  }
  public boolean windowWillShow(Window win){
    if ((win.title() != null) && (windows.get(win.title()) == null))
      windows.put(win.title(), win);
    return true;
  }
  public void windowWillSizeBy(Window win, Size size){}
  /* -------------------------------------------------- */
	
}

