/*
 *
 * Written By: 	Mike Parks
 * Date:			6/29/96
 * Description:	Block Class
 *
 * Usage Notes:
 *
 * 
 */
 /*
 *
 * Copyright (c) 1996 Michael Parks. All Rights Reserved.
 *
 * 
 */ 
  
package vt.cs.virtSchool.labapplets.BlockAndPlane;

import java.awt.*;
import vt.cs.shareablePrimitive.*;
import SharedData.*;

public class Block extends MovingBody {

  Polygon BlockPoly = null;
  float v0=0; // Initial Velocity
  float tc=0; // Critical time that the block moves from one surface to another
  float nextv0; // Iniital Velocity for next plane to use
  float nexttc; // Time constant for next "change of surface"

  // Block size
  //	float length,height;
  protected ShareableFloat height = null;
  protected ShareableFloat length = null;

  // Currently inclination
  float angle;
  // initial position for the block (in pixels)
  float x0[] = null;
  float y0[] = null;
  // Current position for the block (in pixels)
  int xarr[] = new int[4];
  int yarr[] = new int[4];
	
  public Block(float ht,float ln, BlockAndPlane bnp) {
    this(ht, ln, bnp, null, null, "NoName");
  }

  public Block(float ht, float ln, BlockAndPlane bnp,
	       Session session, Client client, String name) {
    super(bnp, session, client, name);
    int deltax,deltay; 
    // default size of block in meters
    /*
      height = ht;
      length = ln;
      */
    height = new ShareableFloat(name+"Height", session, client, ht);
    length = new ShareableFloat(name+"Length", session, client, ln);
    plane = oldplane = null;
    v0 = 0;     // reset intial velocity
    nextv0 = 0; 
    nexttc = 0;
  }   	

  public void setHeight(float NewHeight) {
    height.setValue(NewHeight);
  }
  public float getHeight() {
    return height.floatValue();
  }
  
  public void setLength(float NewLength) {
    length.setValue(NewLength);
  }
  
  public float getLength() {
    return length.floatValue();
  }

  public void ComputeInitialPos(Dimension d) {
    // Computes the initial position of the block
    // Recomputed each time initial position of block is reset
    // and is used for determining movement
    
    debug("ComputeInitialPos()");
    
    int deltay,deltax; 
    float tempx, tempy;
    x0 = new float[4];
    y0 = new float[4];
    
    // Move block to starting position
    plane = (bnp.getPlane1());
    // Now setup initial position for block
    deltax = plane.LeftEnd;
    deltay = (int)(d.height-(SimCanvas.ScaleFactor*plane.getHeight()));
    // bottom left
    x0[0] = (int)(plane.LeftEnd);
    y0[0] = (int)(d.height-(SimCanvas.ScaleFactor*plane.getHeight()));
    // top left
    x0[1] = (int)(plane.LeftEnd);
    y0[1] = (int)(d.height-(SimCanvas.ScaleFactor*(plane.getHeight()+getHeight())));
    // top right
    x0[2] = (int)(plane.LeftEnd+(SimCanvas.ScaleFactor*getLength()));
    y0[2] = (int)(d.height-(SimCanvas.ScaleFactor*(plane.getHeight()+getHeight())));
    // bottom right
    x0[3] = (int)(plane.LeftEnd+(SimCanvas.ScaleFactor*getLength()));
    y0[3] = (int)(d.height-(SimCanvas.ScaleFactor*plane.getHeight()));
    // Shift to (0,0)
    for (int i=0;i<4;i++) {
      x0[i] = x0[i] - deltax;
      y0[i] = y0[i] - deltay;
    }
    // Rotate
    for (int i=0;i<4;i++) {
      tempx = x0[i];
      tempy = y0[i];
      x0[i] = (int)Math.round(tempx*Math.cos(plane.getAngle()) - 
			      (tempy*Math.sin(plane.getAngle())) );
      y0[i] = (int)Math.round(tempx*Math.sin(plane.getAngle()) + 
			      (tempy*Math.cos(plane.getAngle())) );
    }
    // Shift away from (0,0)
    for (int i=0;i<4;i++) {
      x0[i] = x0[i] + deltax;
      y0[i] = y0[i] + deltay;
    }	
    for (int i=0;i<4;i++) {
      xarr[i] = Math.round(x0[i]);
      yarr[i] = Math.round(y0[i]);
    }	
    // Record angle of inclination
    angle = plane.getAngle();
    // Set up CM coordinates
    x = (xarr[0] + xarr[2])/2;
    y = (yarr[0] + yarr[2])/2;
    // Convert to meters
    x = x/SimCanvas.ScaleFactor;
    y = y/SimCanvas.ScaleFactor; 
    // Reset boundry values
    plane = oldplane = null;
    v0 = 0;     // reset intial velocity
    nextv0 = 0; 
    nexttc = 0;
  }

  public void rotate(float angle, Plane plane,int x0[],int y0[]) {
    int deltay, deltax;
    int tempx,tempy;
    // Now setup initial position for block
    deltax = x0[0];
    deltay = y0[0];
    // Shift to (0,0)
    for (int i=0;i<4;i++) {
      x0[i] = x0[i] - deltax;
      y0[i] = y0[i] - deltay;
    }
    // Rotate
    for (int i=0;i<4;i++) {
      tempx = x0[i];
      tempy = y0[i];
      x0[i] = (int)Math.round(tempx*Math.cos(angle) - (tempy*Math.sin(angle)) );
      y0[i] = (int)Math.round(tempx*Math.sin(angle) + (tempy*Math.cos(angle)) );
    }
    // Shift away from (0,0)
    for (int i=0;i<4;i++) {
      x0[i] = x0[i] + deltax;
      y0[i] = y0[i] + deltay;
    }	
  }
  public void computeCM() {
    // Update CM
    x = (xarr[0] + xarr[2])/2;
    y = (yarr[0] + yarr[2])/2;
    // Convert to meters
    x = x/SimCanvas.ScaleFactor;
    y = y/SimCanvas.ScaleFactor; 
  }
  public void move(float t) {
    // Figure out which surface block is on	
    if ( (bnp.getPlane1()).inside(x) ) {
      MovePlane1(t);
      // See if surface transition occured and rectify moving body if necessary
      computeCM();
      if (!(bnp.getPlane1()).inside(x)) {
	a = (-1)*(getGravity() * (bnp.getFloor()).getMuK());
	yarr[0] = yarr[3] = D.height;
	yarr[1] = yarr[2] = D.height-(int)(SimCanvas.ScaleFactor*getHeight());
	xarr[0] = xarr[1] = Math.round(SimCanvas.ScaleFactor*
				       ((0.5f)*a*(t-nexttc)*(t-nexttc)+nextv0*(t-nexttc) + 
					(bnp.getFloor()).getLeftEnd()));
	xarr[2] = xarr[3] = Math.round(xarr[0] + SimCanvas.ScaleFactor*getLength());
      }
      computeCM();
      return;
    }
    if ( (bnp.getFloor()).inside(x)  ) {
      MoveFloor(t);
      computeCM();
      if ((bnp.getPlane2()).inside(x)) {
	// Rectify block
	float angle = (-1)*(bnp.getPlane2()).getAngle();
	float m = ((bnp.getPlane2()).getHeight()/(bnp.getPlane2()).getLength());
	yarr[0] = (int)(D.height-(m*(xarr[0]-(SimCanvas.ScaleFactor*
					      (bnp.getPlane2()).getLeftEnd()))));
	yarr[1] = (int)(yarr[0] - (SimCanvas.ScaleFactor*getHeight()));
	yarr[2] = yarr[1];
	yarr[3] = yarr[0];
	// Now rotate block level with plane
	rotate(angle,(bnp.getPlane2()),xarr,yarr);
      }
      if ((bnp.getPlane1()).inside(x)) {
	// Rectify block
	float angle = (bnp.getPlane1()).getAngle();
	float m = ((bnp.getPlane1()).getHeight()/(bnp.getPlane1()).getLength());
	float diff = SimCanvas.ScaleFactor*((bnp.getPlane1()).getRightEnd());
	diff = (diff - xarr[0]);
	yarr[0] = (int)(D.height-(m*diff));
	yarr[1] = (int)(yarr[0] - (SimCanvas.ScaleFactor*getHeight()));
	yarr[2] = yarr[1];
	yarr[3] = yarr[0];
	// Now rotate block level with plane
	rotate(angle,(bnp.getPlane1()),xarr,yarr);
      }
      computeCM();
      return;
    }
    if ( (bnp.getPlane2()).inside(x) ) {
      MovePlane2(t);
      debug("1 - x="+x);
      computeCM();
      debug("2 - x="+x);

      // this is a total kludge.   - Bo 4/30/97
      if (x < 3.0f)
	x = 3.0f;

      if (!(bnp.getPlane2()).inside(x)) {

      debug("1d - a="+a);
	a = (getGravity() * (bnp.getFloor()).getMuK());	   
      debug("2d - a="+a);

	yarr[0] = yarr[3] = D.height;
	yarr[1] = yarr[2] = D.height-(int)(SimCanvas.ScaleFactor*getHeight());           
	xarr[0] = xarr[1] = Math.round(SimCanvas.ScaleFactor*
				       ((0.5f)*a*(t-nexttc)*(t-nexttc)+v0*(t-nexttc) + 
					(bnp.getFloor()).getRightEnd()));          
	xarr[0] = xarr[1] = Math.round(SimCanvas.ScaleFactor*
				       ((0.5f)*a*(t-nexttc)*(t-nexttc) + nextv0*(t-nexttc) + 
					(bnp.getFloor()).getRightEnd()));       
	xarr[2] = xarr[3] = Math.round(xarr[0] + SimCanvas.ScaleFactor*getLength());
      }
      computeCM();
      return;
    }
  }
  public void MovePlane1(float t) {
    plane = bnp.getPlane1();
    // Check to see if we just got to plane1
    if (oldplane != plane) {
      oldplane = plane;
      v0 = nextv0; // Use precomputed initial velocity
      tc = nexttc;
      // Compute velocity for next plane to use
      if (v0 == 0) { // Only runs on initial start
	float deltah = (getY_CM()-(getHeight()/2));
	nextv0 = (float)Math.sqrt(2*getGravity()*(deltah-plane.getMuK()*plane.getLength())); 

      debug("1e - a="+a);
	a = (float)(getGravity()*(Math.sin(plane.getAngle())-(bnp.getPlane1()).getMuK()*
			     Math.cos(plane.getAngle())));
      debug("2e - a="+a);
	nexttc = tc + (float)Math.sqrt(2*plane.getSurfaceLength()/a);
      }
    }
    if (angle != plane.getAngle()) {   
      angle = plane.getAngle();
      if (v0 != 0) {
	computeCM();      
	float deltax = (plane.getRightEnd() - x);
	float deltay = (float)(deltax*Math.tan(plane.getAngle()));
	deltax = deltax * SimCanvas.ScaleFactor;
	deltay = deltay * SimCanvas.ScaleFactor;
	for (int i=0;i<4;i++) {
	  x0[i] = (xarr[i]+deltax);
	  y0[i] = (yarr[i]+deltay);
	}
	// Use v as temp variable to compute current velocity
	a = (float)(getGravity()*(Math.sin(plane.getAngle())+
			     (bnp.getPlane1()).getMuK()*Math.cos(plane.getAngle())));
	nexttc = (float)(tc+(-v0/a));
	v = (xarr[0]+xarr[2])/2;
	v = v/SimCanvas.ScaleFactor;
	v = (float)Math.sqrt(v0*v0 - 2*plane.getMuK()*getGravity()*v);
      }
    }
    // If no movement, then halt
    if (plane.getMuS() >= Math.tan(plane.getAngle())) {
      v = 0;
      return;
    }
    if ((t > nexttc) && (v0 != 0)) { // Reverse Direction
      computeCM();
      float deltah = (getY_CM()-(getHeight()/2));
      float deltal = (float)(getX_CM()-plane.getRightEnd());
      float deltax = (float)Math.sqrt(deltah*deltah + deltal*deltal);
      for(int i=0;i<4;i++) { // Reset x0 and y0
	x0[i] = (int)Math.round(SimCanvas.ScaleFactor*Math.cos(angle)*0.5f*a*(nexttc-tc)*
				(nexttc-tc) + SimCanvas.ScaleFactor*v0*(nexttc-tc)*
				Math.cos(angle) + x0[i]);
	y0[i] = (int)Math.round(SimCanvas.ScaleFactor*Math.sin(angle)*0.5f*a*(nexttc-tc)*
				(nexttc-tc) + SimCanvas.ScaleFactor*v0*(nexttc-tc)*
				Math.sin(angle) + y0[i]);
      }
      tc = nexttc;      
      a = (float)(getGravity()*(Math.sin(plane.getAngle())-
			   plane.getMuK()*Math.cos(plane.getAngle())));	 
      nexttc = tc + (float)Math.sqrt(2*deltax/a);
      nextv0 = (float)(a*Math.sqrt(2*deltax/a));
      v0 = 0;
    }      
    v = a*(t-tc) + v0;
    for(int i=0;i<4;i++) {
      xarr[i] = (int)Math.round(SimCanvas.ScaleFactor*Math.cos(angle)*0.5f*a*(t-tc)*(t-tc) + 
				SimCanvas.ScaleFactor*v0*(t-tc)*Math.cos(angle) + x0[i]);
      yarr[i] = (int)Math.round(SimCanvas.ScaleFactor*Math.sin(angle)*0.5f*a*(t-tc)*(t-tc) + 
				SimCanvas.ScaleFactor*v0*(t-tc)*Math.sin(angle) + y0[i]);
    }
  }
  public void MoveFloor(float t) {
    plane = (bnp.getFloor());
    // Check to see if we just moved onto the floor
    if (oldplane != plane) {
      oldplane = plane;
      v0 = nextv0; // Use precomputed initial velocity
      tc = nexttc;
      // Compute velocity for next plane to use
      nextv0 = (float)Math.sqrt(v0*v0-2*plane.getMuK()*getGravity()*plane.getLength());
      if (v0 < 0) nextv0 = (nextv0 * (-1)); // FIX FOR FRICTION CASE
      // Friction must always oppose velocity
      if (v0>0) 
	a = (-1)*(getGravity() * plane.getMuK());
      else 
	a = (getGravity() * plane.getMuK());	   
      // Set time constant
      if (a==0) {
	nexttc = tc+(plane.getLength()/Math.abs(v0));
      }
      else {
	nexttc = (float)(tc+((-Math.abs(v0)+Math.sqrt(v0*v0+2*a*plane.getLength()))/a));
      }
    }
    angle = plane.getAngle();
    yarr[0] = yarr[3] = D.height;
    yarr[1] = yarr[2] = D.height-(int)(SimCanvas.ScaleFactor*getHeight());
    v = a*(t-tc) + v0;
    if (v/v0 < 0)  // v cannot change signs on floor
      v = 0;
    if (v>0) {
      xarr[0] = xarr[1] = Math.round(SimCanvas.ScaleFactor*((0.5f)*a*(t-tc)*(t-tc) +
							    v0*(t-tc) + plane.getLeftEnd()));
      xarr[2] = xarr[3] = Math.round(xarr[0] + SimCanvas.ScaleFactor*getLength());
    }
    if (v<0) {
      xarr[0] = xarr[1] = Math.round(SimCanvas.ScaleFactor*((0.5f)*a*(t-tc)*(t-tc) + 
							    v0*(t-tc) + plane.getRightEnd()));
      xarr[2] = xarr[3] = Math.round(xarr[0] + SimCanvas.ScaleFactor*getLength());
    }
    float x = (xarr[0] + xarr[2])/2;
    x = x / SimCanvas.ScaleFactor;
  }
  public void MovePlane2(float t) {
    float m;
    plane = bnp.getPlane2();
    // Check to see if we just moved from the floor (again)
    if (oldplane != plane) {
      oldplane = plane;
      v0 = nextv0; // Use precomputed initial velocity
      tc = nexttc;
      debug("v0="+v0+" tc="+tc);

      debug("1b - a="+a);
      a = (float)((-1)*getGravity()*(Math.sin(plane.getAngle())+
				plane.getMuK()*Math.cos(plane.getAngle())));  
      debug("2b - a="+a);

      // This nexttc for "turnaround point" at top of block
      nexttc = (float)(tc+(-v0/a));
    }
    // Set the block on the plane
    if (angle != (-1)*plane.getAngle()) {
      angle = (-1)*plane.getAngle();
      // Record initial position
      computeCM();
      float deltax = (x - plane.getLeftEnd());
      float deltay = (float)(deltax*Math.tan(plane.getAngle()));
      deltax = deltax * SimCanvas.ScaleFactor;
      deltay = deltay * SimCanvas.ScaleFactor;
      for (int i=0;i<4;i++) {
	x0[i] = (xarr[i]-deltax);
	y0[i] = (yarr[i]+deltay);
      }
      // Set acc and initial velocity
      debug("1a - a="+a);
      a = (float)((-1)*getGravity()*(Math.sin(plane.getAngle())+
				plane.getMuK()*Math.cos(plane.getAngle())));
      debug("2a - a="+a);

      // Use v as a temp variable to compute current velocity
      v = (xarr[0] + xarr[2])/2;
      v = v/SimCanvas.ScaleFactor;
      v = (float)Math.sqrt(v0*v0 - 2*plane.getMuK()*getGravity()*v);
      debug("a="+a+" v="+v);
    }
    // Moving Body at highest point of ascent, changes direction here
    if ((t>nexttc) && (v0 != 0)) {
      computeCM();
      float deltah = (getY_CM()-(getHeight()/2));
      float deltal = (float)(getX_CM()-plane.getLeftEnd());
      float deltax = (float)Math.sqrt(deltah*deltah + deltal*deltal);
      for(int i=0;i<4;i++) { // Reset x0 and y0
	x0[i] = (int)Math.round(SimCanvas.ScaleFactor*Math.cos(angle)*0.5f*a*
				(nexttc-tc)*(nexttc-tc) + SimCanvas.ScaleFactor*v0*
				(nexttc-tc)*Math.cos(angle) + x0[i]);
	y0[i] = (int)Math.round(SimCanvas.ScaleFactor*Math.sin(angle)*0.5f*a*
				(nexttc-tc)*(nexttc-tc) + SimCanvas.ScaleFactor*v0*
				(nexttc-tc)*Math.sin(angle) + y0[i]);
      }
      tc = nexttc;

      debug("1 - a="+a);
      a = (float)((-1)*getGravity()*(Math.sin(plane.getAngle())-
				plane.getMuK()*Math.cos(plane.getAngle())));
      debug("2 - a="+a);
	 
      nexttc = tc + (float)Math.sqrt(2*deltax/-a);
      nextv0 = (float)(a*Math.sqrt(2*deltax/-a));
      v0 = 0;
    }
    // Move block normally
      debug("1 - a="+a+" v="+v);
    v = a*(t-tc) + v0;
      debug("2 - a="+a+" v="+v);

    if (v<=0)
      // If no movement, then return
      if (plane.getMuS() >= Math.tan(plane.getAngle())) {
	return;
      }
    for(int i=0;i<4;i++) {
      xarr[i] = (int)Math.round(SimCanvas.ScaleFactor*
				Math.cos(angle)*0.5f*a*(t-tc)*(t-tc) + 
				SimCanvas.ScaleFactor*v0*(t-tc)*Math.cos(angle) + x0[i]);
      yarr[i] = (int)Math.round(SimCanvas.ScaleFactor*
				Math.sin(angle)*0.5f*a*(t-tc)*(t-tc) + 
				SimCanvas.ScaleFactor*v0*(t-tc)*Math.sin(angle) + y0[i]);
    }
  }

  public void paint(Graphics g, Dimension d) {
    //    debug("paint()");
    if (reset) { 
      reset = false; 
      ComputeInitialPos(d); 
    }
    if (x0 == null) ComputeInitialPos(d);
    if ((D == null) || (D.width != d.width) || (D.height != d.height)) 
      D = d;		
    BlockPoly = new Polygon(xarr,yarr,4);
    g.setColor(Color.cyan);
    g.fillPolygon(BlockPoly);
    g.setColor(Color.white);
  }

  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  }
}
