/*
 *
 * Written By:   James "Bo" Begole
 * Date:         Feb 20 1997
 * Description:  Consumer for a channel that recieves block and plane control 
 *                   event messages.
 *                   These messages will be converted to calls to a
 *                   BlockAndPlane object.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.1 1996/12/13 21:43:54 begolej Exp $
 *
 * Copyright (c) 1996 James Bo Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package vt.cs.virtSchool.labapplets.BlockAndPlane;

import SharedData.*;

/**
 * Consumer for a channel that recieves block and plane control 
 * event messages.
 * These messages will be converted to calls to a
 * BlockAndPlane object.
 *
 * @version $Revision  $
 */
public class ControlChannelConsumer implements ChannelConsumer {
  private Client client = null;
  private Channel channel = null;
  private BlockAndPlane bnp = null;

  public ControlChannelConsumer(Client client, Channel channel,
				BlockAndPlane bnp) {
    this.client = client;
    this.channel = channel;
    this.bnp = bnp;

    try {
      channel.setConsumer(client, this);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /* ------- ChannelConsumer Interface methods ------ */
  public void dataReceived(Data data) {
    BnPMessage msg = null;
    
    if (bnp != null) {
      msg = (BnPMessage)BnPMessage.messageFromData(data);
      switch(msg.type) {
      case BnPMessage.RUN:
	bnp.runSimulation(false);
	break;
      case BnPMessage.PAUSE:
	bnp.pauseSimulation();
	break;
      case BnPMessage.RESET:
	bnp.resetSimulation();
	break;
      }
    }
  }
}
