/*
 *
 * Written By:		James "Bo" Begole
 * Date:			18 June 1996
 * Description:	Block and Plane Simulation Applet
 *
 * Usage Notes:
 *
 * 
 */ 
 
/*
 *
 * Copyright (c) 1996 James "Bo" Begole, All Rights Reserved.
 *
 * 
 */
package vt.cs.virtSchool.labapplets.BlockAndPlane; 

import java.applet.Applet;
import java.awt.*;
import vt.cs.widgets.*;
import vt.cs.virtSchool.util.*;
import vt.cs.db.*;
import SharedData.*;
 
public class BlockAndPlane extends Applet implements Client {
  private static boolean inAnApplet = true;  
  public SimCanvas simCanvas = null;
  public float t_incr = 0.10f;
  public  int   RunCount = 0;
  SimulationThread thread = null;
  boolean Started = false; // Has "Run" been pushed?
  public  ButtonCanvas BC = null;
  public  Plane plane1 = null;
  public  Plane plane2 = null;
  public  Plane floor = null;
  public  MovingBody MB = null;
  public  PlaneDialog Plane1Dialog = null;
  public  PlaneDialog Plane2Dialog = null;
  public  FloorDialog floorDialog = null;		
  public  MovingBodyDialog MBDialog = null;
  public  ConstantDialog CDialog = null;
  public  DataCollectDialog DataCDialog = null;
  Button Button_plane1,Button_plane2,Button_floor,
    Button_config,Button_Data_collect,Button_const,Button_run,Button_pause,Button_reset;
  Choice MovingBodyType = null;		 
  TextField RunCounter = null; 
  TextArea OutputArea1 = null;
  TextArea OutputArea2 = null;

  private DatabaseSet	dbSet   = null;
  private Database    	db  	= null;
  private Table       	table	= null;
  private static final String[] columnNames = 
  {"Distance", "Velocity", "Acceleration", "Time"};
  private static final String[] columnTypes = 
  {"java.lang.Long","java.lang.Long","java.lang.Long","java.lang.Long"};
  private static final String[] columnDescs = 
  {"meters","meters/millisec","meters/millisec/millisec","millisec"};

  // Shared Data variables
  private Session session = null;
  private Channel controlChannel = null;
  private ControlChannelConsumer controlChannelConsumer = null;
  private String name = null;


  public void init() {
    try {
    name =  "Block And Plane"+
      java.net.InetAddress.getLocalHost().getHostName()+
      System.currentTimeMillis();
    } catch (java.net.UnknownHostException uhex) {
      name =  "Block And Plane" + System.currentTimeMillis();
    }
    debug("name = "+name);

    // init canvas objects
    plane1 = new Plane(0.5f,1.0f,0.0f,0.00f,0.02f,0,true);
    plane2 = new Plane(1.0f,2.0f,0.0f,0.00f,0.02f,500,false);
    floor  = new Plane(0f,2.0f,0.0f,0.0f,0f,100,true);
    MB = (MovingBody)(new Block(0.15f,0.15f, this));
    simCanvas = new SimCanvas(plane1,plane2,floor,MB);	 	
    // Set Applet layout
    setLayout(new BorderLayout());
    BC = new ButtonCanvas();
    Panel ButtonCanvasPanel = new Panel();
    ButtonCanvasPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    ButtonCanvasPanel.add(BC);
    Panel PlanePanel = new Panel();
    PlanePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Button_plane1 = new Button("Plane 1");
    PlanePanel.add(Button_plane1);
    Button_floor = new Button("Floor");
    PlanePanel.add(Button_floor);
    Button_plane2 = new Button("Plane 2");
    PlanePanel.add(Button_plane2);
    Panel MBPanel = new Panel(); 
    MBPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Button_config = new Button("Moving Body");
    MBPanel.add(Button_config);
    Panel DataPanel = new Panel();
    DataPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Button_Data_collect = new Button("Data Collection");
    DataPanel.add(Button_Data_collect);
    Panel ConstPanel = new Panel();
    ConstPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Button_const = new Button("Constants");
    ConstPanel.add(Button_const);
    Panel TopPanel1 = new Panel();
    TopPanel1.setLayout(new GridLayout(0,1));
    Label l = new Label("Configure:",Label.LEFT);
    MovingBodyType = new Choice();
    MovingBodyType.addItem("Block");
    MovingBodyType.addItem("Sphere");
    MovingBodyType.addItem("Matchbox Car");
    TopPanel1.add(l); 
    TopPanel1.add(ButtonCanvasPanel);
    TopPanel1.add(PlanePanel);
    TopPanel1.add(MovingBodyType);          
    TopPanel1.add(MBPanel);
    TopPanel1.add(ConstPanel);
    TopPanel1.add(DataPanel);		

    Panel TopPanel2 = new Panel();
    TopPanel2.setLayout(new GridLayout(0,1));
    OutputArea1 = new TextArea(5,20);
    OutputArea1.setEditable(false);
    TopPanel2.add(OutputArea1);
          
    Panel TopPanel3 = new Panel();
    TopPanel3.setLayout(new GridLayout(0,1));
    OutputArea2 = new TextArea(5,20);
    OutputArea2.setEditable(false);
    TopPanel3.add(OutputArea2);
		
    Panel TopPanel = new Panel();
    TopPanel.setLayout(new GridLayout(1,3));
    TopPanel.add(TopPanel1);
    TopPanel.add(TopPanel2);
    TopPanel.add(TopPanel3);
		
    add("North",TopPanel);
		
    // 		Create Simulation Canvas
    simCanvas.setBackground(Color.black);
		
    add("Center",simCanvas);
		
    Panel BottomPanel = new Panel();
    BottomPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
    RunCounter = new TextField(new Integer(RunCount).toString(),3);
    RunCounter.setEditable(false);
    BottomPanel.add(RunCounter);
    Button_run = new Button("Run");
    BottomPanel.add(Button_run);
    Button_pause = new Button("Pause");
    BottomPanel.add(Button_pause);
    Button_reset = new Button("Reset");
    BottomPanel.add(Button_reset);
		
    add("South",BottomPanel);
		
    refresh();

    /*
     * We should have gotten strings identifying our DatabaseSet
     * and Database as applet parms. We'll get references to those
     * here, but we won't create a table until the Run button
     * is pressed.
     */
    String dbSetLocation = getParameter("DatabaseSet");
    if (dbSetLocation != null) {
      try {
	dbSet = Manager.getDatabaseSet(dbSetLocation);
	db = dbSet.getDatabase(getParameter("Database"));
	if (db != null) {
	  String[] tables = db.getTables();
	  RunCount = tables.length;
	  RunCounter.setText(new Integer(RunCount).toString());
	}
      }
      catch (Exception e) {
	e.printStackTrace();
      }
    }

    /*
     * See if we have a sessionURL.  If so, we are in a collaborative
     * session and need to behave accordingly.
     */
    String sessionURL = getParameter("SessionURL");
    if (sessionURL != null) {
      debug("sessionURL "+sessionURL);
      //   sessionURL="coll://simon.cs.vt.edu:4445/socket/Session/BnP2876";
      // Causes following:
      /*
TCPSocketServer: exception java.net.SocketException: Address already in use
TCPSocketServer: run: exception java.lang.NullPointerException
       */
      debug("New sessionURL "+sessionURL);
      try {
	session = (Session)SharedData.Naming.lookup(sessionURL);
      } catch (SharedDataException sde) {
	if (sde.getType() == SharedDataException.NOT_BOUND) {
	  try {
	    session = SessionFactory.createSession(sessionURL);
	    debug("session created: "+sessionURL);
	  } catch (Exception e) {
	    System.out.println("Error creating session: "+e);
	    e.printStackTrace();
	  }
	} else {
	    System.out.println("Error creating session: "+sde);
	    sde.printStackTrace();
	}
      }
      debug("after createSession");
      if (session != null){
	debug("Session successfully created");
	try {
	  session.join(this);
	  debug("joined session "+sessionURL);
	  // We should really get the "ControlChannel" via 
	  // getParameter, so we can change the channel name
	  // Also, if it's null, leave the session and set session=null
	  session.createChannel("ControlChannel", true);
	  debug("Channel successfully created");
	} catch (Exception sdex) {
	  // this channel may already exist.  If so, great.
	  if (sdex instanceof SharedDataException){
	    if (((SharedDataException)sdex).getType() 
		!= SharedDataException.CHANNEL_EXISTS){
	      sdex.printStackTrace();
	    }
	  }
	}
	try {
	  // create controlChannel
	  controlChannel = Channel.join(session, "BlockAndPlaneControl", this);
	  debug("channel successfully joined.");
	} catch (SharedDataException sdex) {
	  System.out.println("Error joining channel: "+sdex);
	  sdex.printStackTrace();
	}
	// add controlChannelConsumer to controlChannel
	if (controlChannel != null)
	  controlChannelConsumer = new ControlChannelConsumer(this, 
							      controlChannel, 
							      this);
	debug("end of shared data initilization.");
      }
    }
  }

  private Frame getParentFrame() {
    Component comp = this;
    while (!(comp instanceof Frame)) {
      comp = comp.getParent();
    }
    return (Frame)comp;
  }

  public  void  refresh() {
    debug("refresh()");
    // Set the block redraw flag
    MB.Reset();
    // Recompute the relative plane positions
    debug("refresh() setfloors and planes");
    floor.SetLeftEnd(plane1.GetLength()+plane1.GetLeftEnd());
    plane2.SetLeftEnd(floor.GetLength()+floor.GetLeftEnd());
    BC.repaint();	   
    simCanvas.repaint();
  }

  public  float getTimeStep() {
    return t_incr;
  }
  public  void setTimeStep(float t) {
    t_incr = t;
  }
  public  Plane getPlane1() {
    return (Plane)plane1;
  }
  public  Plane getPlane2() {
    return (Plane)plane2;
  }
  public  Plane getFloor() {
    return (Plane)floor;
  }

  public boolean action(Event e, Object arg) {
    Object target = e.target;
    if (target == Button_run) {
      debug("run pre notifyParticipants");
      notifyParticipants(BnPMessage.RUN, null);
      debug("run post notifyParticipants");
      runSimulation(true);
      debug("run post pauseSimulation");
      return true;
    }
    if (target == Button_plane1) {
      Plane1Dialog = new PlaneDialog(plane1, getParentFrame(), this);
      Plane1Dialog.show();
      return(true);
    }
    if (target == Button_plane2) {
      Plane2Dialog = new PlaneDialog(plane2, getParentFrame(), this);
      Plane2Dialog.show();
      return(true);
    }
    if (target == Button_config) {
      MBDialog = new MovingBodyDialog((Block)MB, getParentFrame(), this);
      MBDialog.show();
      return(true);
    }
    if (target == Button_Data_collect) {
      DataCDialog = new DataCollectDialog(getParentFrame(), this);
      DataCDialog.show();
      return(true);
    }
    if (target == Button_const) {
      CDialog = new ConstantDialog(MB, getParentFrame(), this);
      CDialog.show();
      return(true);
    }
    if (target == Button_floor) {
      floorDialog = new FloorDialog(floor, getParentFrame(), this);
      floorDialog.show();
      return(true);
    }
    if (target == Button_pause) {
      debug("pause pre notifyParticipants");
      notifyParticipants(BnPMessage.PAUSE, null);
      debug("pause post notifyParticipants");
      pauseSimulation();
      debug("pause post pauseSimulation");
      return(true);
    }

    if (target == Button_reset) {
      debug("reset pre notifyParticipants (resetSimulation)");
      notifyParticipants(BnPMessage.RESET, null);
      debug("reset post notifyParticipants(resetSimulation)");
      resetSimulation();
      debug("reset post resetSimulation");      return(true);
    }
    return false;
  }

  public void runSimulation(boolean iPressedRun) {
    debug("runSimulation begin");
      Started = true;
      Button_run.disable();
      resetSimVars();
      if (thread == null) {
	RunCount+=1;
	RunCounter.setText(new Integer(RunCount).toString());

	if ((db != null) && (iPressedRun)){
	  table = db.newTable("Run " + 
			      RunCounter.getText(),
			      columnNames, columnTypes, columnDescs,
			      "Block and Plane Applet");
	}

	RecordUserSettings();
	thread = new SimulationThread(simCanvas, MB, this, t_incr);
	thread.start();
      }
      else {
	thread.resume();			   
      }
    debug("runSimulation end");
  }

  public void pauseSimulation() {
    debug("pauseSimulation begin");
      if (thread != null) {
	thread.suspend();
	Button_run.setLabel("Resume");
	Button_run.enable();
	Button_run.resize(Button_run.preferredSize());
	validate();
      }
    debug("pauseSimulation end");
  }

  public void resetSimulation() {
    debug("resetSimulation begin");
      if (thread != null) {
	Button_run.enable();
	Button_run.setLabel("Run");
	Button_run.resize(Button_run.preferredSize());
	validate();
	Started = false;
	thread = null;
	table = null;
	MB.Reset();
	simCanvas.repaint();
      }
    debug("resetSimulation end");
  }

  private void  notifyParticipants(int msgType, Object data) {
      if (controlChannel != null) {
	try {
	  controlChannel.uniformSendToOthers(this, 
                  new Data(BnPMessage.toByteArray(
			     new BnPMessage(msgType, data))));
	} catch (Exception sdex) {
	  sdex.printStackTrace();
	}
      }
  }

  public void RecordUserSettings() {
    OutputArea1.appendText("\n");   
    OutputArea1.appendText("RUN "+RunCount+"\n");   
    OutputArea1.appendText("----------\n"); 	
    OutputArea2.appendText("RUN "+RunCount+"\n");   
    OutputArea2.appendText("----------\n"); 
    addHeader("Plane1 Length",new Float(plane1.GetLength()).toString());
    addHeader("Plane1 Height",new Float(plane1.GetHeight()).toString());
    addHeader("Plane1 Angle",new Float(plane1.GetAngle()).toString());
    addHeader("Plane1 MuS",new Float(plane1.GetMuS()).toString());
    addHeader("Plane1 MuK",new Float(plane1.GetMuK()).toString());
    addHeader("Plane2 Length",new Float(plane2.GetLength()).toString());
    addHeader("Plane2 Height",new Float(plane2.GetHeight()).toString());
    addHeader("Plane2 Angle",new Float(plane2.GetAngle()).toString());
    addHeader("Plane2 MuS",new Float(plane2.GetMuS()).toString());
    addHeader("Plane2 MuK",new Float(plane2.GetMuK()).toString());
    addHeader("Floor Length",new Float(floor.GetLength()).toString());
    addHeader("Floor MuS  ",new Float(floor.GetMuS()).toString());
    addHeader("Floor MuK",new Float(floor.GetMuK()).toString());
    addHeader("Block Length",new Float(((Block)MB).GetLength()).toString());
    addHeader("Block Height",new Float(((Block)MB).GetHeight()).toString());
    addHeader("Time Step    ",new Float(t_incr).toString());
    addHeader("Gravity      ",new Float(MB.GetGravity()).toString());
    OutputArea2.appendText("\n");   	   
  }         
  public void addHeader(String ParamName,String Value) {
    OutputArea2.appendText(ParamName+"\t"+Value+"\n");
    if (table != null) {
      table.setAttribute(ParamName, Value);
    }
  }

  float oldX = 0.0f;   //previous position
  float oldY = 0.0f;   // previous position
  float oldTime = 0.0f;
  float deltaTime = 0.0f;
  double deltaDistance = 0.0f;
  double distance = 0.0f;
  double oldDistance = 0.0f;
  double velocity = 0.0f;
  double deltaVelocity = 0.0f;
  double oldVelocity = 0.0f;
  double acceleration = 0.0f;
  int timeCounter = 0;

  private void resetSimVars() {
    oldX = 0.0f;   //previous position
    oldY = 0.0f;   // previous position
    oldTime = 0.0f;
    deltaTime = 0.0f;
    deltaDistance = 0.0f;
    distance = 0.0f;
    velocity = 0.0f;
    deltaVelocity = 0.0f;
    oldVelocity = 0.0f;
    acceleration = 0.0f;
    timeCounter = 0;
  }

  public void addRecord(float x,float y,float time) {
    timeCounter++;
    
    deltaDistance = Math.sqrt((x-oldX)*(x-oldX) + (y-oldY)*(y-oldY));
    //deltaDistance = distance-oldDistance;
    //oldDistance = distance;
    oldX = x;
    oldY = y;

    distance += Math.abs(deltaDistance);

    deltaTime = t_incr;
    if (timeCounter > 1) 
      velocity = deltaDistance/deltaTime;
    
    deltaVelocity = velocity-oldVelocity;
    oldVelocity = velocity;
    if (timeCounter > 2)
      acceleration = (deltaVelocity)/deltaTime;
    
    /* debug(" distance="+distance+
		       " velocity="+velocity+
		       " acceleration="+acceleration+
		       " time="+time);
     */

    if (time == 0)
      OutputArea1.appendText("Run\tDist\tVel\tAcc\tTime\n");
    OutputArea1.appendText(RunCount+"\t"+Fmt.fmt(distance,3,3)+
			   "\t"+Fmt.fmt(velocity,3,3)+
			   "\t"+Fmt.fmt(acceleration,3,3)+
			   "\t"+Fmt.fmt(time,3,3)+"\n");
    if (table != null) {
      Object[] row = new Object[4];
      row[0] = new Long((long)(distance*1000));
      row[1] = new Long((long)(velocity*1000));
      row[2] = new Long((long)(acceleration*1000));
      row[3] = new Long((long)(time*1000));
      
      try {
	table.appendRow(row);
      }
      catch (Exception e) {
	e.printStackTrace();
      }
    }
  }

  public float getTimeIncrement() {
    return t_incr;
  }

  public void start() {
    super.start();
  }

  public void stop() {
    super.stop();
  }

  public void destroy() {
    disconnect();
  }

  private void disconnect() {
    try {
      if (session != null) {
	session.leave(this);
	session = null;
      }
      if (controlChannel != null) {
	controlChannel.leave(this);
	controlChannel = null;
      }
    } catch (Exception e) {
      System.err.println("Caught exception while trying to " +
			 "disconnect from Block and Plane: " + e);
      e.printStackTrace();
    }
  }

  protected void finalize() throws Throwable{
    destroy();
    super.finalize();
  }

  /* ----------- Client Interface methods ----------- */
  public String authenticate(String challenge) {
    System.out.println("BlockAndPlane: athenticate: challenge="+challenge);
    return null;
  }
  public String getName() {
    return name;
  }
  public void clientChanged(ClientEvent event) {
        System.out.println("BlockAndPlane: clientChanged. event="+event);
  }
  /* ------------------------------------------------ */

  
  public static void main(String args[]) {
    inAnApplet = false;
    AppletFrame.startApplet("BlockAndPlane", 
			    "Block and Plane Simulator Frame", 
			    args);
  }

  private static  final boolean debug = true;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  }
}



/**
  SimultaionThread handles running the simulation when Run is pressed.
 */
class SimulationThread extends Thread {
  private SimCanvas simCanvas = null;
  private  MovingBody MB = null;
  private BlockAndPlane bnp = null;
  private float timeStep = 0.0f;

  SimulationThread(SimCanvas simCanvas, MovingBody MB, 
		   BlockAndPlane bnp, float timeStep) {
    this.simCanvas = simCanvas;
    this.MB = MB;
    this.bnp = bnp;
    this.timeStep = timeStep;
  }

  public void run() {
    float t = 0;
    while ((SimCanvas.ScaleFactor*MB.GetX_CM() < simCanvas.size().width) && 
	   bnp.Started ) {
      MB.move(t);				     
      bnp.addRecord(MB.GetX_CM(), MB.GetY_CM(), t);
      simCanvas.repaint();
      try {
	sleep((long)Math.round(timeStep*1000));
      } catch(InterruptedException e) {
	System.out.println(e);
      }
      t += timeStep;
    }
  }

}
