/*
** Written By: Hien Do
** Date:       11 November 1996
** Descrip:    A comsumer that takes in an array of double and 
**             outputs it in the form of a graph
*/

package vt.cs.virtSchool.labapplets.Graph;

import vt.cs.db.*;

import java.awt.*;
import java.lang.*;
import java.applet.*;
import java.util.*;

public class Graph extends java.applet.Applet {	

  //graph class that contains many methods, drag box zoom,
  //manually setting the range, etc.
  private G2Dint               graph;

  //x axis of graph
  private Axis                 xaxis;

  //y axis of graph
  private Axis                 yaxis;
  
  //panel to hold the checkbox for graph data
  private Panel        checkBoxPanel;

  //vector of checkbox, contains the checkbox, color, and label
  private Vector      checkBoxVector;

  //Vector of DataSets, contains the data points and attributes
  //such as line color.
  private Vector       dataAttribVec;

  private Legend              legend;

  //displays the table attributes
  //Frame                        attrib;
  
  Choice	           axisChoice;

  MPanel                 gP;

  SelectFrame selection; 

  Label notify = null;

  public void init() {
    String dbsetURL = getParameter("DatabaseSet");
    String dbURL = getParameter("Database");
    String tableURL = getParameter("Table");

    gP = new MPanel(this);

    legend = new Legend("= Legend =");
    legend.setLayout(new GridLayout(1,1));

    setLayout(new BorderLayout());
       
    //System.out.println("dbset: " + dbsetURL);

    Font f;
    f = new Font("Helvetica", Font.PLAIN, 12);
    setFont(f);

    selection = new SelectFrame(dbsetURL, dbURL, tableURL, this);
    if(selection.notnull == false){
      gP.setGraph(null);
      gP.disableButtons();
      notify = new Label("Database set not found.", Label.CENTER);
      f = new Font("TimesRoman", Font.BOLD, 20);
      notify.setFont(f);
      notify.setForeground(Color.red);
      add("Center", notify);
      add("South", gP);      
    }
  }

  public void graphSelectedTable(Table t, String title){
    setupNewData();
    graph.setTitle(title);
    runGraph(t);
  }

  /***** 
  ** sets up the graph attributes
  *****/
  private void graphAttrib(G2Dint gph) {
    gph.borderTop    = 30;
    gph.borderBottom = 10;
    gph.borderRight  = 40;
    gph.setDataBackground(Color.white);
    gph.setGraphBackground(Color.lightGray);
  }
 
  /*****
  ** Show the selection window
  *****/
  public void showSelect(){
    //System.out.println(selection.isShowing());
    //System.out.println(selection);
    if((selection.isShowing() == false) &&
       (selection.notnull == true)){
      selection.resize(300, 200);
      selection.show();
    }
  }

  /*****
  ** Create a new select frame and don't use the applet parameters
  *****/
  public void newSelect(){
    SelectFrame tmp = new SelectFrame(null, null, null, this);
    //System.out.println(tmp.notnull);
    if(tmp.notnull == true){
      if(notify != null){
	remove(notify);
      }
      gP.enableButtons();
      selection = tmp;
    }
  }

  private void runGraph(Table t) { 
    
    Table table = t;
    //System.out.println("table" + table);
    //local variables
    vt.cs.virtSchool.labapplets.Graph.DataSet datain;
    //Vector points;  //vector of data points
    int labelIndex;
    String dataLabel;
    Vector keys = new Vector();
    long starttime; //start time of data collection
    int i; //maps to x values
    int j; //maps to y values
    int k; //loop counter
    
    graphAttrib(graph);
      if (table != null) {
	//System.out.println("table not null");
	for (Enumeration e = table.columnNames("java.lang.Long");
	     e.hasMoreElements(); ) {
	  String name = (String) e.nextElement();
	  keys.addElement(name);
	}
      }
	
      //printTableAttrib(table);
	
      //System.out.println("keysize: " + keys.size());
    checkBoxPanel.setLayout(new GridLayout(keys.size() + 1, 1)); //set layout type for checkBoxPanel
    
    if (keys.size()> 0){ //check to see if the Vector keys has data
      Vector independent = null;

      try {
        // independent = table.getColumn((String)keys.elementAt(0));

        independent = 
           table.getColumn((String)keys.elementAt(keys.size()-1));
      }
      catch (Exception e) {
        e.printStackTrace();
      }

      for (labelIndex = 0; labelIndex < keys.size(); labelIndex++){
        dataLabel = (String)keys.elementAt(labelIndex);
        axisChoice.addItem(dataLabel);
        //System.out.println("Using stream: " + dataLabel);
	
        //System.out.println("table rowcount: " + table.rowcount());
        double data[] = new double[2*(int)table.rowcount()];
	
        try {	
      	  long numRows = table.rowcount();
          Vector dependent = table.getColumn(dataLabel);
          for (i=j=0; i < numRows; i++, j+=2) {
            data[j+1] = ((Number)(dependent.elementAt(i))).doubleValue();
            data[j] = ((Number)(independent.elementAt(i))).doubleValue();

            //data[j+1] = ((Number)table.get(i, dataLabel)).doubleValue();
            //data[j] = ((Number)table.get(i, 
            //   (String)keys.elementAt(keys.size()-1)) ).doubleValue();
	  }
	} catch(DatabaseException e){
	  System.out.println("Caught DatabaseException: " + e);
	}

	//setTitle("Graphing data for "+ table.getName());
	
	//sets up graph attributes for data

	datain = graph.loadDataSet(data, (int)table.rowcount());
	datain.linestyle = 1;
	datain.linecolor = LegendColor.getSomeColor(labelIndex);
	datain.marker    = 1;
	datain.markerscale = 1.5;
	datain.markercolor = new Color(0, 0, 255);
	
	dataAttribVec.addElement(datain);
	
	if (labelIndex == 0){ //create the X and Y axis only once through the loop
	  xaxis = graph.createXAxis();  //creates X axis at bottom of graph
	  //xaxis.setTitleText("Xaxis");             //label of x axis
	  xaxis.setTitleColor(Color.blue);         //text label color
	  xaxis.setLabelColor(new Color(170, 0, 255)); //marker label color
	  
	  yaxis = graph.createYAxis();   //creates Y axis at the left of the graph
	  //yaxis.setTitleText("Y Axis");
	  yaxis.setTitleColor(Color.red); 
	  yaxis.setLabelColor(Color.black);
	}
	
	xaxis.attachDataSet(datain); //attach appropriate data to x and y
	yaxis.attachDataSet(datain);
	
	Checkbox temp;
	if(labelIndex == 0){
	  temp = new Checkbox(dataLabel, null, true);
	}
	else{
	  temp = new Checkbox(dataLabel, null, false);
	  graph.hideDataSet(datain);
	}
	checkBoxPanel.add(temp);
	temp.setForeground(LegendColor.getSomeColor(labelIndex));
	checkBoxVector.addElement(temp);
      }  
 
      gP.setGraph(graph);
      
      add("South", gP);
      add("Center", graph);
      validate(); //validate applet
      axisChoice.select(keys.size()-1);
      xaxis.setTitleText(axisChoice.getSelectedItem());
      checkBoxPanel.add(axisChoice);
      
      legend.setReferences(graph, checkBoxVector, dataAttribVec, 
	axisChoice, table, xaxis, yaxis);
      legend.resize(200, 50*(checkBoxVector.size() + 1));
      legend.add(checkBoxPanel);
      legend.show();
    }
    else{
      //setTitle("Could not locate data at specified location");
      setLayout(new FlowLayout());
      add(new Label("No data to graph!"));

      Button newDataBtn = new Button("   New Data   ");
      newDataBtn.setBackground(Color.blue);
      newDataBtn.setForeground(Color.white);
      add(newDataBtn);
    }
  }

  /*
  private void printTableAttrib(Table tbl) {
    
    Vector attribkeys = new Vector();
    Label temp;
    Table table = tbl;
    
    attrib = new Frame();
    Panel atp = new Panel();
    Panel sbp = new Panel();
    
    attrib.setTitle("Attributes");
    
    try {
      //System.out.println("Attributes for " + table.getName());
      for (Enumeration e = table.attributeKeys(); e.hasMoreElements();) {
	String tablekey = (String) e.nextElement();
	//System.out.println("	" + tablekey + " = " + (String) table.getAttribute(tablekey));
	String str = tablekey + " = " + (String) table.getAttribute(tablekey);
	attribkeys.addElement(str);
      }
    }
    catch (Exception e) { e.printStackTrace();}
    
    atp.setLayout(new GridLayout(attribkeys.size(), 1));
    for (int i=0; i < attribkeys.size(); i++) {
      temp = new Label((String)attribkeys.elementAt(i));
      atp.add(temp);
    }
    attrib.setLayout(new BorderLayout());
   
    attrib.add("Center", atp);
    //attrib.add("West", sbp);
    attrib.resize(200, 200);
    attrib.show();
    
  }
  */
 
  /*****
   ** Called when the applet is started or revisited
   *****/
  public void start(){
    if(legend != null){
      legend.show();
    }
  }

  /*****
   ** Called when the applet is stopped.
   *****/
  public void stop(){
    if(legend != null){
      legend.hide();
    }
  }
 
  private void setupNewData() {
    /*
    if(attrib != null){
      if(attrib.isShowing()){
	attrib.hide();
      }
      remove(attrib);
    }
    */

    //hide legend if it's showing and remove the check box panel that it contains
    //in order to set up for new data
    if(legend != null){
      if(legend.isShowing()){
	legend.hide();
      }
      if(checkBoxPanel != null){
	legend.remove(checkBoxPanel);
      }
    }

     // initialize class variables
    //attrib = null;
    xaxis = null;
    yaxis = null;

    if(graph != null){
      //System.out.println("remove current graph");
      graph.hide();
      remove(graph);
      graph = null;
    }
    checkBoxPanel = null;
    checkBoxVector = null;
    dataAttribVec = null;
    axisChoice= null;

    graph = new G2Dint();
    checkBoxPanel = new Panel();
    checkBoxVector = new Vector();
    dataAttribVec = new Vector();
    axisChoice = new Choice();
  }

/***** 
 ** Overrides the minimumSize method.
 *****/
public synchronized Dimension minimumSize(){
  Dimension dim = new Dimension(500, 400);
  return(dim);
}

  /*****
  ** Overrides the perferredSize method.
  *****/
  public Dimension preferredSize(){
    return minimumSize();
  }
}
