package vt.cs.virtSchool.labapplets.Graph;

import java.awt.*;
import java.applet.*;
import java.util.*;
import java.lang.*;
import java.io.StreamTokenizer;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;

/*************************************************************************
**
**    Class  LoadData
**                                              Version 1.0   December 1995
**                                              Version 1.1   February 1996 
**
**************************************************************************
**    Copyright (C) 1995, 1996 Leigh Brookshaw
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**************************************************************************
**
**    This class will load data from a given URL
**
*************************************************************************/




public class LoadData extends Thread {

/*******************
** Public Variables
*******************/



/********************
** Private Variables
********************/

/*
**  Place we can send error/informational messages
*/
   private Graph2D graph = null;
   private Applet applet = null;
/*
** The URL where we are to get the data
*/
    private URL file;
/*
**  Number of data points in temporary storage.  
*/
   private int Max  = 250;

/*
** The DataSet we will load the data into
*/
   private DataSet ds = null;
/*
** Optional array to load data into
*/
   private double array[] = null;
/*
** Boolean - has the data load finished
*/
   private boolean finished = false;
/*
** Boolean - has the data load started
*/
   private boolean started = false;
   
/*
** number of data points.
*/
   private int count = 0;


/***************
** Constructors
****************/

    public LoadData() {
           finished = false;
           started     = false;
	 }

    public LoadData(DataSet d) {
           ds = d;
           finished = false;
           started     = false;
	 }

/**************
** Public Methods
**************/


    public DataSet loadDataSet( URL file ) {
           if(file == null) return null;
           if(ds == null) ds = new DataSet();

           this.file = file;
           this.start();
           return ds;
    }
    
    public void loadArray( URL file ) {
           if(file == null) return;
           this.file = file;
           this.start();
    }

    public DataSet loadDataSet( URL file, Object drawable ) {    
           if(file == null) return null;
           if(ds == null) ds = new DataSet();
           this.file     = file;
           if(drawable != null) {
              if(drawable instanceof Applet ) {
                       applet = (Applet)drawable;
              } else 
              if(drawable instanceof Graph2D) {
                       graph = (Graph2D)drawable;
                       graph.attachDataSet(ds);
              }
           }        
           this.start();
           return ds;
    }
    
    public void loadArray( URL file, Object drawable ) {    
           if(file == null) return;
           this.file     = file;
           if(drawable != null) {
              if(drawable instanceof Applet ) {
                       applet = (Applet)drawable;
              } else 
              if(drawable instanceof Graph2D) {
                       graph = (Graph2D)drawable;
                       graph.attachDataSet(ds);
              }
           }        
           this.start();
           return;
    }


    public void run () {
           int datamax = 2*Max;
           byte b[] = new byte[50];
           int nbytes = 0;
           double data[] = new double[datamax];
           int size = 0;

           InputStream is = null;
           boolean comment    = false;
           int c;

           setPriority(Thread.MIN_PRIORITY);

           try {
                 is = file.openStream();
                 
                 started = true;
                 count = 0;

                 if( graph != null )   graph.startedloading();

                 while( (c=is.read()) > -1 ) {
                     
                     switch (c) {
                     
                         case '#':
                                    comment = true;
                                    break;
                         case '\r': case '\n':
                                    comment = false;
                         case ' ': case '\t':
                                if( nbytes > 0 ) {
                                   String s = new String(b,0,0,nbytes);
                                   data[size] = Double.valueOf(s).doubleValue();
                                   size++;
                                   if( size >= datamax ) {
                                       append(data, size);
                                       size = 0;
                                       System.out.println(
                                         "Loading Data! Points Loaded "+
                                          count);
                                   }
                                   nbytes = 0;
                                }
                                break;
                         default:
                                   if( !comment ) {
                                        b[nbytes] = (byte)c;
                                        nbytes++;
                                   }
                                   break;
                      }

                  }

             if( size > 1 )  append(data, size);

             if (is != null) is.close();
        } catch(Exception e) {
          printmessage("Error loading data!");
          return;
        }
        
        
        started  = false;
        finished = true;
        
        if( graph != null) graph.finishedloading();

        if( count == 0 ) {
                        printmessage("Zero data loaded!");
                        return;
                      }




    }

    public DataSet getDataSet() { return ds; }
    public synchronized double[] getArray()  { return array; }

    public boolean started()  { return started;  }
    public boolean finished() { return finished; }
    
    public int length() { return count; }
    
    private synchronized void append(double data[], int size) {
    
      if(size == 0) return;
      count += size;

      if( ds != null) {
         try {  ds.append(data, size/2); }
         catch (Exception e) {
             printmessage("Failed to append data to DataSet!");
	   }
      } else 
      if(array != null) {
               double tmp[] = new double[array.length + size];
               System.arraycopy(array,0,tmp,0,size);
               System.arraycopy(data,0,tmp,array.length,size);
               array = tmp;
      } else {
               array = new double[size];
               System.arraycopy(data,0,array,0,size);
      }
            
      return;     
    }



    private void printmessage( String s) {

         if(s==null)        return;

         if( applet != null )   applet.showStatus(s);

         System.out.println(s);


    }




}
