
//
//	Circle.java
//
//	Copyright 1996,	Black Diamond Software.	 All rights reserved.
//
//	Written	by: Jason Beaver and Tyler Gingrich
//

package	vt.cs.virtSchool.labapplets.WhiteBoard;

import netscape.application.*;
import netscape.util.*;

public class Circle extends Graphic {
	Point center;
	int radius;

	// Anchor points...
	private	static final int CENTER	= 0;

	public Circle()	{
		super();
	}

	public Circle(Point aPoint,int aRadius)	{
		super();

		center = aPoint;
		radius = aRadius;
	}

	// Called to set the receiver up for creating...
	public void createGraphic(MouseEvent event,SimpleDrawView view)	{
		super.createGraphic(event,view);

		center = new Point(event.x,event.y);
		radius = 0;
		anchor = CENTER;
	}

	// This	is called repeatedly as	the user manipulates the graphic...
	public void updateGraphic(MouseEvent event,SimpleDrawView view)	{
		super.updateGraphic(event,view);

		if (view.currentOperation() == SimpleDrawView.CREATING ||
			view.currentOperation()	== SimpleDrawView.RESIZING) {
			switch (anchor)	{
				case CENTER:
					try { radius = (int)Math.sqrt(Math.pow(event.x-center.x,2)+Math.pow(event.y-center.y,2)); }
					catch (ArithmeticException e) {	}
					break;
			}
		}
		view.graphicChanged(this);
	}

	// This	is called whenever the graphic should display itself...
	public void drawGraphic(Graphics g) {
		super.drawGraphic(g);

		if (isBeingManipulated()) {
			g.drawLine(center.x-2, center.y-2, center.x+2, center.y+2);
			g.drawLine(center.x+2, center.y-2, center.x-2, center.y+2);
		}
		if (isFilled())	{
			g.setColor(fillColor());
			g.fillArc(center.x-radius,center.y-radius,2*radius,2*radius,0,360);
		}
		g.setColor(strokeColor());
		g.drawArc(center.x-radius,center.y-radius,2*radius,2*radius,0,360);
	}

	// Returns a Vector of the graphic's handles (as Points)...
	public Vector handles()	{
		Point upper, lower, right, left;
		Point lowerLeft, lowerRight, upperLeft,	upperRight;
		int   delta = (int)(radius * Math.sin(Math.PI/4));

		Vector handles = new Vector();

		lowerLeft = new	Point(center.x-delta, center.y-delta);
		handles.addElement(lowerLeft);

		lowerRight = new Point(center.x+delta, center.y-delta);
		handles.addElement(lowerRight);

		upperLeft = new	Point(center.x-delta, center.y+delta);
		handles.addElement(upperLeft);

		upperRight = new Point(center.x+delta, center.y+delta);
		handles.addElement(upperRight);

		lower =	new Point(center.x, center.y-radius);
		handles.addElement(lower);

		left = new Point(center.x-radius, center.y);
		handles.addElement(left);

		upper =	new Point(center.x, center.y+radius);
		handles.addElement(upper);

		right =	new Point(center.x+radius, center.y);
		handles.addElement(right);

		return handles;
	}

	// Returns the anchor corresponding to the hit handle...
	public int anchorForHitHandle(MouseEvent event,SimpleDrawView view) {
		return CENTER;
	}

	// Used	for hit	detection.  This algorithm is crude.  It simply	uses the
	// circle's bounding rectangle...
	public boolean intersectsRect(Rect aRect) {
		Rect comparisonRect = new Rect(center.x-radius,center.y-radius,2*radius,2*radius);

		if(comparisonRect.width	<1) comparisonRect.width = 1;
		if(comparisonRect.height <1) comparisonRect.height = 1;

		if (comparisonRect.intersects(aRect)) {
			//  If in here,	do some	fancy stuff for	hit detection
			//  but	for now, just punt.

			return true;
		}

		return false;
	}

	public void moveBy(int deltaX, int deltaY) {
		center.moveBy(deltaX,deltaY);
	}
}
