
//
//	SimpleDrawView.java
//
//	Copyright 1996,	Black Diamond Software.	 All rights reserved.
//
//	Written	by: Jason Beaver and Tyler Gingrich
//

package	vt.cs.virtSchool.labapplets.WhiteBoard;

import netscape.application.*;
import netscape.util.*;
import java.rmi.*;
import vt.cs.virtSchool.server.*;
import SharedData.*;

public class SimpleDrawView extends View implements DragDestination,
	Client, ChannelConsumer {

	DrawController 	drawController;
	Vector 			graphics;
	Graphic			currentGraphic;
	int 			currentOperation;
	Point 			lastMouseLocation;

	Hashtable		graphicsIDs	= null;
	String			roomURL		= null;
	RoomData		room		= null;
	Session			roomSession	= null;
	Channel			wbChannel	= null;
	String			name		= null;

	// Operations...
	public static final int	NONE = -1;
	public static final int	CREATING = 0;
	public static final int	RESIZING = 1;
	public static final int	MOVING = 2;

	public SimpleDrawView(DrawController aController,int x,int y,int width,
		int height, String roomURL) {
		super(x,y,width,height);

		drawController = aController;
		graphics = new Vector();
		graphicsIDs = new Hashtable();
		currentOperation = NONE;

		this.roomURL = roomURL;
		name = new String("_Whiteboard" + System.currentTimeMillis());
		try {
			room = (RoomData) java.rmi.Naming.lookup(roomURL);
			roomSession = SessionFactory.createSession(room.getSessionURL());
			roomSession.join(this);
			wbChannel = Channel.join(roomSession, RoomData.WHITEBOARD_CHANNEL,
				this);
			wbChannel.setConsumer(this, this);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void drawView(Graphics g) {
		Vector selectedGraphics	= selectedGraphics();

		// Erase the background...
		g.setColor(Color.white);
		g.fillRect(bounds());
		System.out.println("SimpleDrawView bounds()"+bounds());

		// Draw	all the	graphics...
		for (int i=0,count = graphics.count();i<count;i++)
			((Graphic)graphics.elementAt(i)).drawGraphic(g);

		// Draw	all the	selected graphic's handles...
		for (int i=0,count = selectedGraphics.count();i<count;i++)
			((Graphic)selectedGraphics.elementAt(i)).drawHandles(g);
	}

	// Return all the graphics...
	public Vector graphics() {
		return graphics;
	}

	// Return the selected graphics. We build this Vector dynamically because
	// it is important that	the selected graphics are in the Vector	in the same
	// order they are in graphics...
	public Vector selectedGraphics() {
		Vector selectedGraphics	= new Vector();

		for (int i=0,count=graphics.count();i<count;i++) {
			Graphic	graphic	= ((Graphic)graphics.elementAt(i));

			if (graphic.isSelected())
				selectedGraphics.addElement(graphic);
		}

		return selectedGraphics;
	}

	public void addGraphic(Graphic graphic)	{
		
		graphics.addElement(graphic);
	}

	public void removeGraphic(Graphic graphic) {
		graphics.removeElement(graphic);
	}

	public void selectAllGraphics()	{
		for (int i=0,count=graphics.count();i<count;i++)
			selectGraphic((Graphic)graphics.elementAt(i));
	}

	public void unselectAllGraphics() {
		for (int i=0,count=graphics.count();i<count;i++)
			unselectGraphic((Graphic)graphics.elementAt(i));
	}

	public void selectGraphic(Graphic graphic) {
		graphic.setSelected(true);
	}

	public void unselectGraphic(Graphic graphic) {
		graphic.setSelected(false);
	}

	public void updateInspector() {
		drawController.updateInspector();
	}

	public void setCurrentOperation(int operation) {
		currentOperation = operation;
	}

	public int currentOperation() {
		return currentOperation;
	}

	public Color currentStrokeColor() {
		return drawController.currentStrokeColor();
	}

	public Color currentFillColor()	{
		return drawController.currentFillColor();
	}

	public boolean currentFilledState() {
		return drawController.currentFilledState();
	}

	// Called when a graphic has changed.  Currently we just draw the entire view
	// which is very inefficient.  You could modify	this to	draw just the affected
	// portion of the view for more	efficient drawing...
	public void graphicChanged(Graphic graphic) {
//System.out.println("in SimpleDrawView.graphicChanged(): " + graphic);
		draw();
	}

	// Handles the mouse down event...
	public boolean mouseDown(MouseEvent event) {
		String tool = drawController.selectedTool();


		// Make	sure we're the focused view so we receive key events...
		rootView().setFocusedView(this);

		if (tool.equals("Pointer")) {
			// If the current tool is the pointer, we've got lots to
			// do, so let's	get started...
			Rect pointerRect = new Rect(event.x-1,event.y-1,3,3);
			Point hitPoint = new Point(event.x, event.y);

			Vector selectedGraphics	= selectedGraphics();

			// If the shift	key is down, add/remove	graphics from the selection...
			if (event.isShiftKeyDown()) {
				for (int i=graphics.count()-1;i>=0;i--)	{
					Graphic	graphic	= ((Graphic)graphics.elementAt(i));

					if (graphic.intersectsRect(pointerRect)) {
						if (graphic.isSelected())
							unselectGraphic(graphic);
						else
							selectGraphic(graphic);
						graphicChanged(graphic);
						return true;
					}
				}
			}
			// See if we hit a selected graphic's handle. If so, start resizing it...
			for (int i=(selectedGraphics.count()-1);i>=0;i--) {
				Graphic	graphic	= ((Graphic)selectedGraphics.elementAt(i));

				if (graphic.hitHandle(event)) {
					currentOperation = RESIZING;
					currentGraphic = graphic;
					currentGraphic.resizeGraphic(event,this);
					return true;
				}

			}
			// See if we hit a selected graphic. If	so, start moving all selected graphics...
			for (int i=graphics.count()-1;i>=0;i--)	{
				Graphic	graphic	= ((Graphic)graphics.elementAt(i));

				if (graphic.intersectsRect(pointerRect)) {
					if (!graphic.isSelected())
						break;
					currentOperation = MOVING;
					currentGraphic = graphic;
					lastMouseLocation = new	Point(event.x,event.y);
					return true;
				}
			}
			// Otherwise, unselect all the graphics...
			unselectAllGraphics();
			// If we hit a graphic,	select it and start moving it...
			for (int i=graphics.count()-1;i>=0;i--)	{
				Graphic	graphic	= ((Graphic)graphics.elementAt(i));

				if (graphic.intersectsRect(pointerRect)) {
					selectGraphic(graphic);
					currentOperation = MOVING;
					currentGraphic = graphic;
					lastMouseLocation = new	Point(event.x,event.y);
					return true;
				}
			}
			// If we STILL haven't done anything, we've hit	the background.
			// Create a lasso and start lassoing...
			if (currentOperation ==	NONE) {
				currentGraphic = new Lasso();
				currentOperation = CREATING;
				currentGraphic.createGraphic(event,this);
				return true;
			}
		} else {
			// If the current tool is not the pointer, find	it's class...
			Class toolClass;
			try { toolClass	= Class.forName(
				"vt.cs.virtSchool.labapplets.WhiteBoard."+tool); }
				catch (ClassNotFoundException e) { return false; }

			// And try to instanciate one...
			try { currentGraphic = (Graphic)(toolClass.newInstance()); }
				catch (InstantiationException e1) { return false; }
				catch (IllegalAccessException e2) { return false; }

			// If we did, unselect all graphics and	select our new one...
			unselectAllGraphics();
			selectGraphic(currentGraphic);
			currentOperation = CREATING;
			// Now start the creation...
			currentGraphic.createGraphic(event,this);

			String id = null;

			id = new String(currentGraphic.toString() + "." + 
				System.currentTimeMillis());
			graphicsIDs.put(id, currentGraphic);
			currentGraphic.setID(id);

			try {
System.out.println("Adding graphic ID: " + id);
				WhiteBoardMessage msg = 	
					new WhiteBoardMessage(WhiteBoardMessage.WHITEBOARD_ADD, 
					id, tool, currentOperation, event.x, event.y);
				wbChannel.uniformSendToAll(this, msg.toData());
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			return true;
		}
		return false;
	}

	// Called when the mouse is dragged (after an initial mouse down)...
	public void mouseDragged(MouseEvent event) {
		switch (currentOperation) {
			case MOVING:
				// Figure the move delta...
				Point currentMouseLocation = new Point(event.x,event.y);
				int deltaX = currentMouseLocation.x-lastMouseLocation.x;
				int deltaY = currentMouseLocation.y-lastMouseLocation.y;
				lastMouseLocation = currentMouseLocation;

				// Move	all the	selected graphics...
				Vector selectedGraphics	= selectedGraphics();
				for (int i=0,count=selectedGraphics.count();i<count;i++) {
					Graphic	graphic	= ((Graphic)selectedGraphics.elementAt(i));

					graphic.moveBy(deltaX,deltaY);

					try {
						if (graphic.id() != null) {
							WhiteBoardMessage msg = 	
								new WhiteBoardMessage(
								WhiteBoardMessage.WHITEBOARD_MOVE, 
								graphic.id(), "", currentOperation,
								deltaX, deltaY);
							wbChannel.uniformSendToAll(this, msg.toData());
						}
					}
					catch (Exception e) {
						e.printStackTrace();
					}

				}
				draw();

				break;
			case CREATING:
			case RESIZING:
				currentGraphic.updateGraphic(event,this);
				break;
		}
	}

	// Called when the mouse button	is released (after an initial mouse down)...
	public void mouseUp(MouseEvent event) {
		if (currentGraphic != null) {
			/*
			 * Let the other clients know we're finished. Should we wait
			 * and show the graphic when the message is received? Could
			 * two clients "fight" over it?
			 */
			try {
				String tool = drawController.selectedTool();
				if (currentGraphic.id() != null) {
					WhiteBoardMessage msg = 	
						new WhiteBoardMessage(
							WhiteBoardMessage.WHITEBOARD_UPDATE, 
							currentGraphic.id(), tool, currentOperation,
							event.x, event.y);
					wbChannel.uniformSendToAll(this, msg.toData());
				}
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			// If we have a	graphic, conclude any manipulation...
			currentGraphic.concludeGraphic(event,this);

		}
		// Sync	up the inspector...
		updateInspector();
		currentOperation = NONE;
		currentGraphic = null;
	}

	// Called when the user	presses	a key.	We only	care about deletes...
	public void keyDown(KeyEvent event) {
		if (event.isBackspaceKey()) {
			Vector selectedGraphics	= selectedGraphics();

			for (int i=selectedGraphics.count()-1;i>=0;i--) {
				Graphic	graphic = (Graphic)selectedGraphics.elementAt(i);
				removeGraphic(graphic);

				try {
					WhiteBoardMessage msg = 	
						new WhiteBoardMessage(
							WhiteBoardMessage.WHITEBOARD_REMOVE, 
							graphic.id(), "", currentOperation, 0, 0);
					wbChannel.uniformSendToAll(this, msg.toData());
				}
				catch (Exception e) {
					e.printStackTrace();
				}

			}
			draw();
		}
	}

	public DragDestination acceptsDrag(DragSession session,	int x, int y) {
		return (session.data() instanceof Color)?this:super.acceptsDrag(session,x,y);
	}

	public boolean dragEntered(DragSession session)	{
		return true;
	}

	public boolean dragMoved(DragSession session) {
		return true;
	}

	public void dragExited(DragSession session) {
	}

	RootView rootView = null;
/*  public void setRootView(RootView rootView) {
    this.rootView = rootView;
  }
*/

	// Called when the user	drops something	on our view...
	public boolean dragDropped(DragSession session)	{
		if (!(session.data() instanceof	Color))	return false;

		Color color = (Color)session.data();

		RootView rootView = Application.application().mainRootView();
		Point dropPoint	= rootView.convertPointToView(this, session.absoluteMousePoint());

		Rect dropPointRect = new Rect (dropPoint.x, dropPoint.y, 1, 1);

		for(int	i= graphics.count()-1;i>=0;i--)	{
			Graphic	graphic	= ((Graphic) graphics.elementAt(i));
			int		msgType	= -1;

			if(graphic.intersectsRect(dropPointRect)) {
				// If control key is down, set stroke color, else set fill color...
				if (session.isControlKeyDown())	{
					if (!graphic.acceptsDraggedStrokeColor()) return false;
					graphic.setStrokeColor(color);
					msgType = WhiteBoardMessage.WHITEBOARD_SET_STROKE_COLOR;
				} else {
					if (!graphic.acceptsDraggedFillColor())	return false;
					graphic.setFilled(true);
					graphic.setFillColor(color);
					msgType = WhiteBoardMessage.WHITEBOARD_SET_FILL_COLOR;
				}
				try {
					WhiteBoardMessage msg = 	
						new WhiteBoardMessage( msgType, graphic.id(), 
							color.rgb());
					wbChannel.uniformSendToAll(this, msg.toData());
				}
				catch (Exception e) {
					e.printStackTrace();
				}

				draw();
				updateInspector();
				return true;
			}
		}
		return false;
	}

	/* ----------- Client Interface methods ----------- */
    public String authenticate(String challenge) {
        return null;
    }
    public String getName() {
        return name;
    }
    public void clientChanged(ClientEvent event) {
    }
    /* ------------------------------------------------ */

    /* ------- ChannelConsumer Interface methods ------ */
    public void dataReceived(Data data) {
		WhiteBoardMessage	msg						= null;
		Graphic				origCurrentGraphic 		= currentGraphic;
		int					origCurrentOperation 	= currentOperation;
		MouseEvent			mouseEvent 				= new MouseEvent();
		Graphic				graphic 				= null;

		/*
		 * Don't process messages from ourself.
		 */
		if (data.getSenderName().equals(name)) {
			return;
		}

		msg = WhiteBoardMessage.messageFromData(data);

		mouseEvent.x = msg.x;
		mouseEvent.y = msg.y;

		if (msg.id != null) 
			graphic = (Graphic) graphicsIDs.get(msg.id);

		if (msg.type == WhiteBoardMessage.WHITEBOARD_ADD) {
			System.out.println("Got ADD");

			if (! graphicsIDs.containsKey(msg.id)) {
				Class toolClass;
				try { 
					toolClass = Class.forName( 
						"vt.cs.virtSchool.labapplets.WhiteBoard."+msg.tool); 
				}
				catch (ClassNotFoundException e) { 
					e.printStackTrace();
					return;
				}

				try { currentGraphic = (Graphic)(toolClass.newInstance()); }
					catch (InstantiationException e1) { return; }
					catch (IllegalAccessException e2) { return; }

				// If we did, unselect all graphics and	select our new one...
				//unselectAllGraphics();
				//selectGraphic(currentGraphic);
				//currentOperation = CREATING;
				// Now start the creation...
				currentGraphic.setID(msg.id);
				currentGraphic.createGraphic(mouseEvent, this);
				graphicsIDs.put(msg.id, currentGraphic);
			}
		}
		else if (msg.type == WhiteBoardMessage.WHITEBOARD_UPDATE) {
			System.out.println("Got UPDATE");
		
			currentGraphic = graphic;
			currentOperation = msg.operation;
			if (currentGraphic != null) {
				currentGraphic.updateGraphic(mouseEvent,this);

				currentGraphic = (Graphic) graphicsIDs.get(msg.id);
				currentOperation = msg.operation;
				currentGraphic.concludeGraphic(mouseEvent,this);
			}
			else
				System.out.println("Couldn't find graphic id = " + msg.id);
		}
		else if (msg.type == WhiteBoardMessage.WHITEBOARD_MOVE) {
			System.out.println("Got MOVE");
			graphic.moveBy(msg.x, msg.y);
		}
		else if (msg.type == WhiteBoardMessage.WHITEBOARD_REMOVE) {
			System.out.println("Got REMOVE");
			if (graphic.isSelected())
				unselectGraphic(graphic);

			removeGraphic(graphic);

			/*
			 * Need to remove graphic from the graphicsIDs table...
			 */
			draw();
		}
		else if (msg.type == WhiteBoardMessage.WHITEBOARD_SET_FILL_COLOR) {
			System.out.println("Got SET FILL COLOR");
			graphic.setFilled(true);
			graphic.setFillColor(new Color(msg.rgb));
			draw();
		}
		else if (msg.type == WhiteBoardMessage.WHITEBOARD_SET_STROKE_COLOR) {
			System.out.println("Got SET STROKE COLOR");
			graphic.setStrokeColor(new Color(msg.rgb));
			draw();
		}
		else
			System.out.println("Got *Unknown* message type");

		currentGraphic = origCurrentGraphic;
		currentOperation = origCurrentOperation;
	}
    /* ------------------------------------------------ */

}
