/*
 *
 * Written By:   James "Bo" Begole
 * Date:         2 July 1996
 * Description:  Remote Object Server implementation
 *
 * Usage Notes:
 *
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/objectManager/ObjectManagerImpl.java,v 1.1.1.1 1996/09/17 05:23:16 isenhour Exp $
 *
 * Copyright (c) 1996 James "Bo" Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
package vt.cs.virtSchool.objectManager;

import java.rmi.RemoteException;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.Naming;
import java.rmi.server.UnicastRemoteServer;
import java.rmi.server.StubSecurityManager;

import java.util.*;
import java.io.*;
import java.net.InetAddress;


public class ObjectManagerImpl 
extends UnicastRemoteServer
implements ObjectManager {
  private static final boolean debug = false;

  private final String objectPath = "vt.cs.virtSchool.util.";
  private final String objectDelimiter = "$";
	
  private  String savedObjectsPath = "./savedObjects/";
  private File savedObjects = new File(savedObjectsPath);
      String name = "ObjectManagerImpl:NoName";


  public ObjectManagerImpl (String name) throws RemoteException {
    super();
    this.name = name;
  }

  public String retrieveNamedObject(String name) throws java.rmi.RemoteException {
    if (debug) System.out.println("ObjectManagerImpl.retrieveNamedObject: "+name);
    // if the named object is in the current list of bound names, return a ref to it.
    if (!isBound(name)) {
      if (debug) System.out.println("ObjectManagerImpl.retrieveNamedObject: Name not bound "+name);
      // Check to see if the object has been saved to a file.
      if (isSaved(name)) {
	if (debug) System.out.println("ObjectManagerImpl.retrieveNamedObject: Name is saved "+name);
	// If so, restore the object and return the reference to it.
	// not implemented - Bo
//	restoreNamedObject(name);

      } else {    
	// Finally, since the named object doesn't exist, create it
	if (debug) System.out.println("ObjectManagerImpl.retrieveNamedObject: Name not saved "+name);
	createNamedObject(name);
      }
    }
	if (debug) System.out.println("ObjectManagerImpl.retrieveNamedObject: Name is now bound "+name);

    return buildURL(name);
  }

  private void createNamedObject(String name) {
    if (debug) System.out.println("ObjectManagerImpl.createNamedObject: "+name);
    String objURL = buildURL(name);
    String className = parseClass(name);

    try {
      Object obj = Class.forName(className).newInstance();
      if (debug) System.out.println("ObjectManagerImpl.createNamedObject(): after object instantiation");
      Naming.rebind(name, (Remote)obj);
    } catch (java.net.UnknownHostException ex) {
      ex.printStackTrace();
      System.out.println("ObjectManagerImpl.createNamedObject(): Host unknown " + objURL);
    } catch (RemoteException ex) {
      ex.printStackTrace();
      System.out.println("ObjectManagerImpl.createNamedObject(): RemoteException during lookup."+objURL);
    } catch (java.net.MalformedURLException ex) {
      ex.printStackTrace();
      System.out.println("ObjectManagerImpl.createNamedObject(): Malformed URL " + objURL);
    } catch (ClassNotFoundException ex) {
      ex.printStackTrace();
      System.out.println("ObjectManagerImpl.createNamedObject(): ClassNotFound " + className);
    } catch (InstantiationException ex) {
      ex.printStackTrace();
      System.out.println("ObjectManagerImpl.createNamedObject(): InstantiationException " + className);
    } catch (IllegalAccessException ex) {
      ex.printStackTrace();
      System.out.println("ObjectManagerImpl.createNamedObject(): IllegalAccessException " + className);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  } 


  private String parseClass(String objectName) {
        StringTokenizer st = new StringTokenizer(objectName, objectDelimiter);
	String className =  new String(objectPath);
	className = className + st.nextToken()+"Impl";
      if (debug) System.out.println("ObjectManagerImpl.parseClass(): className= "+className);	
	return className;
  }


  private boolean isBound(String name) {
    if (debug) System.out.println("ObjectManagerImpl.isBound: "+name);
    String names[] = null;
    try {
      names = Naming.list(name);
    } catch (RemoteException ex) {
      ex.printStackTrace();
      System.out.println("ObjectManagerImpl.isBound: RemoteException during lookup.");
    } catch (java.net.MalformedURLException ex) {
      System.out.println("ObjectManagerImpl.isBound: Malformed URL " + name);
    } catch (java.net.UnknownHostException ex) {
      System.out.println("ObjectManagerImpl.isBound: UnknownHost: " + name+" "+ex);
    }
    if (names != null ) {
      for (int i = 0; i<names.length; i++) {
	if (debug) System.out.println("Bound name: "+names[i]);
	if (names[i].equals("rmi:/"+name))
	  return true;
      }
    }

    return false;
  }

  private boolean isSaved(String name) {
    return false;
    /*
    if (debug) System.out.println("ObjectManagerImpl.isSaved: "+name);
    String names[] = savedObjects.list();

    for (int i = 0; i<names.length; i++) {
      if (names[i].equals(name)) 
	return true;
    }
    return false;
    */
  }

  private String buildURL(String objectName){
    if (debug) System.out.println("ObjectManagerImpl.buildURL: "+objectName);
    String tempURL=null;
    try {
      tempURL = new String("//"+InetAddress.getLocalHost().getHostName()+"/"+objectName);
    } catch (java.net.UnknownHostException ex) {
      System.out.println("Host unknown in where " + ex);
    }
    if (tempURL != null)
      return tempURL;
    else
      return null;
  }


 
 
  public static void main(String args[]) {
    if (debug) System.out.println("ObjectManagerImpl.main: ");
    // Create and install the security manager
    System.setSecurityManager(new StubSecurityManager());

    try {
      String name = "ObjectManager";
      ObjectManagerImpl objectManager = new ObjectManagerImpl(name);
      Naming.rebind(name, objectManager);
      System.out.println("ObjectManager ready.");
            
    } catch (Exception ex) {
      System.out.println("ObjectManagerImpl.main: an exception occurred: " + ex);
    }
  }
}
