/*
 *
 * Written By:   Philip Isenhour
 * Date:         21 December 1996
 * Description:  Consumer for a channel that recieves activity event messages.
 *				 These messages will be converted to calls to a
 *				 ActivityContainer object.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.1 1996/12/13 21:43:54 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package vt.cs.virtSchool.server;

import SharedData.*;

/**
 * Consumer for a channel that recieves activity event messages.
 * These messages will be converted to calls to a
 * ActivityContainer object.
 *
 * @version $Id Exp $
 */
public class ActivityChannelConsumer implements ChannelConsumer {
	private	Client				client		= null;
	private	Channel				channel		= null;
	private ActivityContainer	container	= null;

	public ActivityChannelConsumer(Client client, Channel channel,
		ActivityContainer container) {
		this.client = client;
		this.channel = channel;
		this.container = container;
		try {
			channel.setConsumer(client, this);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	/* ------- ChannelConsumer Interface methods ------ */
	public void dataReceived(Data data) {
		Message         msg     = null;
		Activity        act     = null;
		Link            link    = null;

		if ((container != null)
		    //&& (!data.getSenderName().equals(client.getName()))
		     ) {
			msg = Message.messageFromData(data);

			switch(msg.type) {
				case Message.ACTIVITY_MOVE:
					container.activityMoved((Activity) msg.data);
					break;
				case Message.ACTIVITY_ADD:
					container.activityAdded((Activity) msg.data);
					break;
				case Message.LINK_ADD:
					container.linkAdded((Link) msg.data);
					break;
				case Message.LINK_REMOVE:
					container.linkRemoved((Link) msg.data);
					break;
				default:
					/* Throw an exception here? */
			}
		}
	}
	/* ------------------------------------------------ */

}



