/*
 *
 * Written By:   Philip Isenhour
 * Date:         21 December 1996
 * Description:  Interface for things that contain activities, such as
 *				 the table and shelf views in the client and their
 *               corresponding structures in the server's RoomDataImpl.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.1 1996/12/13 21:43:54 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.server;

/**
 * Interface for things that contain activities, such as
 * the table and shelf views in the client and their
 * corresponding structures in the server's RoomDataImpl.
 *
 * @version $Id Exp $
 */
public interface ActivityContainer {
	/**
	 * Called when an activity is added to the container.
	 */
	public void activityAdded(Activity act);

	/**
	 * Called when an activity is moved. This should perhaps be
	 * generalized to "update" to handle other kinds of changes.
	 */
	public void activityMoved(Activity act);

	/**
	 * Called when an activity is removed from the container.
	 * (Nothing implements this as of December 1996.)
	 */
	public void activityRemoved(Activity act);

	/**
	 * Called when an activity is "claimed" by a client. This should
	 * perhaps be generalized to "opened" to handle activities which
	 * are not restricted to a single user. This is not intended to
	 * handle locking of restricted applets -- that should probably be
	 * done with SharedData.Tokens. (And in that case, the functionality
	 * planned for this method could probably be done by observing the
	 * token rather than sending a message.)
	 * (Nothing implements this as of December 1996.)
	 */
	public void activityClaimed(Activity act);

	/**
	 * Called when an activity is "released" by a client. This should
	 * perhaps be generalized to "closed" to handle activities which
	 * are not restricted to a single user.
	 * (Nothing implements this as of December 1996.)
	 */
	public void activityReleased(Activity act);

	/**
	 * Called when a link is added to the container.
	 */
	public void linkAdded(Link link);

	/**
	 * Called when a link is removed from the container.
	 */
	public void linkRemoved(Link link);
}

