/*
 *
 * Written By:   Philip Isenhour
 * Date:         11 September 1996
 * Description:  Interface for producer->consumer data stream functionality.
 *
 * Usage Notes:
 *
 */

package vt.cs.virtSchool.util;
 
import java.util.*;
import java.applet.Applet;
 
import vt.cs.virtSchool.util.*;
import vt.cs.virtSchool.objectManager.ObjectManager;

/**
 * ConsumerStream is the consumer-side representation of a producer-consumer
 * data stream. Static methods are provided for identify one or more
 * streams of data which the consumer might be able to interpret. The
 * label of one of those streams can then be used to construct a new
 * ConsumerStream instance from which data can be read.
 */
public class ConsumerStream implements Runnable {

	private String	label = null;
	private String	typeName = null;	
	private String	source = null;	

	private int		numRead = 0; 	// The number of elements read
	private Vector	values = null;  // Local copy of all values
	private Thread	reader = null;
	private boolean stopped = false;
	private Lock	readerLock = new Lock();	// Need vector of these
												// for multiple threads? Or
												// just sync get* routines...
	

	// These should go away...
    private static ObjectManager   objectManager = null;
    private static DataSets        dataSets = null;

	static void init(Applet app) {
		if (objectManager == null) {
			objectManager = RemoteObject.getObjectManager(app);
			dataSets = RemoteObject.getDataSets(app, objectManager);
		}
	}

	/**
	 * Returns a vector of strings of labels for all available streams of
	 * the specified type.
	 */
	public static Vector getLabelsByType(Applet app, String typeName) {
		Vector labels = null;
		init(app);
		try {
			labels = dataSets.getKeys(typeName);
		}
		catch (java.rmi.RemoteException e) {
			System.out.println(e);
		}
		return labels;
	}

	// Instance methods from here on...


	/**
	 * Constructs a new ConsumerStream to read values with the specified
	 * label. An appropriate label may be determined using the class-level
	 * (static) methods.
	 */
	public ConsumerStream(String label) {
		this.label = label;
		reader = new Thread(this);
		reader.start();
	}

	public synchronized void run() {
		//while (! stopped) {
		while (true) {
			getVector();
			try {
				reader.sleep(1000);
			}
			catch (InterruptedException e) {
				System.out.println(e);
			}
		}
	}

	synchronized void getVector() {
		Vector vals = null;
		try {
			vals = dataSets.get(label);
		}
		catch (java.rmi.RemoteException e) {
			System.out.println(e);
		}
		if (values == null)
			values = vals;
		else {
			synchronized(values) {
				values = vals;
			}
		}
		readerLock.signal();	// Wake up anyone waiting for data
	}

	/**
	 * Returns a Vector of DataObjects representing all available values 
	 * in the stream. This method never
	 * blocks -- if there's nothing in the stream a null vector is
	 * returned.
	 */
	public Vector getAllValues() {
		if (values == null)
			getVector();

		if (values != null)
			numRead = values.size();

		return values; // Clone this before returning? Need to synchronize?
	}

	/**
	 * Returns the next value in the stream. It will block until data is
	 * available.
	 */
	public DataObject getNext() {
		while ((values == null) || (values.size() <= numRead)) {
			readerLock.block();
		}
		numRead++;
		return (DataObject) values.elementAt(numRead - 1);
	}
}

