/*
 *
 * Written By:   James "Bo" Begole
 * Date:         1 July 1996
 * Description:  Remote Object for Reaction Timer Data
 *
 * Usage Notes:
 *
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/util/DataSetImpl.java,v 1.4 1996/09/17 19:40:48 isenhour Exp $
 *
 * Copyright (c) 1996 James "Bo" Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */


package vt.cs.virtSchool.util;

import java.util.*;

import java.rmi.RemoteException;
import java.rmi.Naming;
import java.rmi.server.UnicastRemoteServer;
import java.rmi.server.StubSecurityManager;

public class DataSetImpl extends UnicastRemoteServer implements DataSet {

  private final boolean debug = false;

  public boolean newData = false;
  private long minPointValue = Long.MAX_VALUE;
  private long maxPointValue = Long.MIN_VALUE;

  private String name = "DataSetImpl:NoName";	

  // PLI ----------------------------------------------------------------------
  Hashtable	vectors = new Hashtable();
  private static final String defaultVectorName = "_default";
  // PLI ----------------------------------------------------------------------

  public DataSetImpl () throws RemoteException {
	super();
	vectors.put(defaultVectorName, new Vector());
  }

  public DataSetImpl (String name) throws RemoteException {
    super();
    this.name = name;
	vectors.put(defaultVectorName, new Vector());
  }

  // PLI ----------------------------------------------------------------------
  /**
   * Gets a vector of objects matching the key.
   * @param key Identifies specific vector of data
   */
  public Vector get(String key) throws RemoteException {
	return (Vector)vectors.get(key);
  }

  /**
   * Gets a vector of objects matching the key and having the specified type.
   * @param key Identifies specific vector of data
   * @param typeName Identifies the name of the type which objects in the vector must have
   */
  public Vector get(String key, String typeName) throws RemoteException {
	Vector vec = (Vector)vectors.get(key);
	/*
	 * If the vector is empty then we don't know if it's elements are
 	 * of the correct type, so we'll return null. (The vector would be
	 * useless anyway?)
	 */
	if ((vec != null) && (vec.size() > 0) && 
		(((DataObject)(vec.firstElement())).className().equals(typeName)))
		return null;
	else
		return vec;
  }

  /**
   * Puts an object into a vector associated with the specified key. If such
   * a vector does not exist then it is created. The object will not be added
   * if a vector with the specified key does exist but contains objects of
   * a different type.
   * @param key Identifies specific vector of data
   * @param value Object to add to the vector
   */
  public synchronized void put(String key, Object value) throws RemoteException{
	Vector vec = (Vector)vectors.get(key);
	if (vec == null) {
		vec = new Vector();
		vec.addElement(value);
		vectors.put(key, vec);
	}
	else {
		if (vec.firstElement().getClass().equals(value.getClass())) 
			vec.addElement(value);
		else	// Throw an exception here?
			System.out.println("Bad type for key " + key);
	}
  }

  /**
   * Returns vector of keys for vectors of objects with the specified type.
   * @param typeName Full name of the type (eg, java.lang.Long)
   */
  public Vector getKeys(String typeName) throws RemoteException {
	Vector keys = new Vector();
	for (Enumeration e = vectors.keys(); e.hasMoreElements();) {
		String	key = (String)e.nextElement();
		if (! key.equals(defaultVectorName)) {
			Vector 	vec = (Vector)vectors.get(key);
			Object	firstObj = vec.firstElement();

			if (firstObj != null)  {
				if (((DataObject)firstObj).className().equals(typeName))
					keys.addElement(key);
			}
		}
	}
	return keys;
  }
  // PLI ----------------------------------------------------------------------
	
  public Vector getPoints()  throws RemoteException {
	Vector defaultVector = (Vector)vectors.get(defaultVectorName);
	return defaultVector;
  }
	
  public synchronized void addPoint(long newPoint) throws RemoteException {
	Vector defaultVector = (Vector)vectors.get(defaultVectorName);
	defaultVector.addElement(new Long(newPoint));

    if (debug) System.out.println("addPoint: start "+newPoint+" Min: "+minPointValue+" Max:"+maxPointValue);
    newData = true;
    if (newPoint > maxPointValue) {
      maxPointValue = newPoint;
    }
    if (newPoint < minPointValue) {
      minPointValue = newPoint;
    }

    if (debug) System.out.println("addPoint: end "+newPoint+" Min: "+minPointValue+" Max:"+maxPointValue);

  }

  public synchronized long maxValue(){
    return maxPointValue;
  }
  public synchronized long minValue() {
    return minPointValue;
  }

  public boolean newData() throws java.rmi.RemoteException {
    return newData;
  }


  public static void main(String args[]) 
  {
    // Create and install the security manager
    System.setSecurityManager(new StubSecurityManager());

    try {
      for(int i=1; i<=5; i++) {
	System.out.println("DataSetImpl.main: create a DataSetImpl");
	String name = "DataSet" + i;
	DataSetImpl ds = new DataSetImpl(name);

	System.out.println("DataSetImpl.main: bind it to name: " + name);
	Naming.rebind(name, ds);
      }
      System.out.println("DataSet Server ready.");
            
    } catch (Exception e) {
      System.out.println("DataSetImpl.main: an exception occurred: " +
			 e.getMessage());
      e.printStackTrace();
    }
  }

  public void setName (String name) throws RemoteException {
    this.name = name;
  }
}


