/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
package vt.cs.widgets;

import java.awt.*;
import java.util.StringTokenizer;
import java.util.Date;

/**
<a href="http://jars.developer.com/classes/jediSearch.cgi?string=RadarPane&matchcase=NO&names=YES&keywords=YES&descriptions=YES&authorid=NOID&lastname=NONAME&age=NOAGE&start=NODATE&end=NODATE&maxhits=25&type=ALL&category=ALL&language=ALL&source=YES&nosource=YES&wholeword=NO&submit=Submit"><IMG SRC=http://simon.cs.vt.edu/~begolej/images/jarsTop5.gif border=0 ></a></p>
 * RadarPane - This is a scrollable panel; a drop-in replacement of <a
href="http://www.javasoft.com/products/JDK/1.1/docs/api/java.awt.ScrollPane.html">java.awt.ScrollPane</a> and
 *  implements the same methods as java.awt.ScrollPane.
 *  RadarPane only works in Java 1.1 or better.</p>
 *
 * RadarPane is available for download at 
 * <a href="http://simon.cs.vt.edu/~begolej/download/RadarPane.jar">
 * http://simon.cs.vt.edu/~begolej/download/RadarPane.jar</a></p>
 * 
 * Contact James &quot;Bo&quot Begole, 
 * <a href="mailto:begolej@vt.edu?subject=RadarPane feedback">begolej@vt.edu</a>, 
 * for bugs fixes and enhancement requests.
 *
<hr>
<h3>Related Work</h3>
RadarPane in use:
<ul>
<dt><a href="http://simon.cs.vt.edu/JAMM"><strong>JAMM: Java Applets Made
Multiuser</strong></a>
<dd>a prototype Java runtime environment that allows the shared use of
single-user Swing-based applets. Includes a <a href="http://java.sun.com/products/jfc/swingdoc-current/index.html">Swing</a>-based version of RadarPane - if you want the <a href="http://java.sun.com/products/jfc/swingdoc-current/index.html">Swing</a> version, let <a href="mailto:begolej@vt.edu?subject=Swing-based RadarPane">me (begolej@vt.edu)</a> know.</p>
<dt><a href="http://simon.cs.vt.edu/sieve"><strong>Sieve</strong></a>
<dd>a Java-based collaborative interactive visualization environment.</p>
</ul>

Tell <a href="mailto:begolej@vt.edu?subject=RadarPane usage">me (begolej@vt.edu)</a> about your use of RadarPane.</p>
 *<hr>
 *
 *<h2>Using RadarPane</h2>
 *  RadarPane includes a radar
 *  view of an embedded component, which is a scaled down image of 
 *  the entire embedded component. 
 *  The figure below shows RadarPane.  The scroll pane is on the
 *  left and the radar view on the right.</p>
 * <img src="http://simon.cs.vt.edu/~begolej/images/RadarPane.gif"></p>
 *
 * Navigate by using the scroll bars or by using the radar view window.
 * To navigate using the window, click in the window.  If, you click outside
 * the viewport rectangel, or click with the right mouse button,
 * the scroll pane
 * will move so that it is centered on your click.
 * If you click inside the viewport rectangle, the viewport will move
 * relative to your moves, until you release the mouse.</p>
 *
 *  The radar view window may be hidden and displayed by clicking 
 *  the checkbox in the lower right of the scroll pane (See figure above).
 *  The radar view component may be detached from the radar view window.
 *  If the radar view is detached, no radar view window or 
 *  checkbox will appear.</p>
 *
 *<hr>
 *<h2>Adding RadarPane to a Program</h2>
 * RadarPane has the same API as java.awt.ScrollPane.  To use it, add() a
 * java.awt.Component to a RadarPane object.  The component will usually be a 
 * java.awt.Panel that contains other components.  
 * The applet pictured above with source code may be found at
 * <a href="http://simon.cs.vt.edu/~begolej/applets/RadarPaneTest/">
 * http://simon.cs.vt.edu/~begolej/applets/RadarPaneTest/</a></p>
 * Here is a sample:</p>
 * <pre>
 * import java.awt.*;
 * import java.applet.*;
 * import vt.cs.widgets.*;
 *  
 * public class RadarPaneTest extends Applet {
 *  
 *   RadarPane scroller;
 *  
 *   public RadarPaneTest() {
 *     setLayout(new BorderLayout());
 *     scroller = new RadarPane(RadarPane.SCROLLBARS_AS_NEEDED);
 *     scroller.add(new ButtonPanel());
 *     add("Center", scroller);
 *   }
 * }
 * 
 * class ButtonPanel extends Panel {
 *   ButtonPanel() {
 *     setLayout(new FlowLayout(FlowLayout.CENTER, 20, 30));
 *     for (int i=0; i<50; i++) 
 *         add(new Button("Button "+i));
 *     resize(getPreferredSize());
 *   }
 *   
 *   public Dimension getMinimumSize() {
 *     return new Dimension(500, 500);
 *   }
 *   
 *   public Dimension getPreferredSize() {
 *     return getMinimumSize();
 *   }
 * }
 * </pre>
 * <h2>Embedding java.awt.Panel Objects</h2>
 * One warning with a workaround.  
 * RadarPane uses the Component.printAll() method to get an image
 * of the embedded component.  If the component is a Container, the printAll()
 * method will print all of the children of that container, 
 * <em>but not the containter itself.</em>  For example, if you embed a Panel
 * that overrides the paint() or update() methods to paint on itself, 
 * those graphics will
 * not appear in the radar view.  The workaround is to override the 
 * Container.print() method of your panel with the following:
 * <pre>
 *   public void print(Graphics g) {
 *     super.print(g);
 *     paint(g);
 *   }
 * </pre>

<hr>
<P>Hits since Feb 01, 1997: <IMG SRC="http://www.bev.net/cgi-bin/counter.xbm?RadarPane" >
</P>
 *
 * @version 	$Revision: 1.23 $
 * @author      James Begole
 */

public class RadarPane 
extends java.awt.Panel 
implements java.io.Serializable, ViewWindowParent {

  private static String revisionString = "$Revision: 1.23 $";
  { // Strip the '$' character from the revisionString
    StringTokenizer st = new StringTokenizer(revisionString, "$");
    revisionString = st.nextToken();
  }

  private void copyright() { 
  }
  /*
  {
    System.out.println(getClass().getName()+": "+revisionString+"\n"+
	    "  (c) copyright 1997, James 'Bo' Begole, All rights reserved.\n"+
	    "  Permission is granted to use this class unmodified.");
  } */
  
  /**
   * Specifies that horizontal/vertical scrollbar should be shown 
   * only when the size of the child exceeds the size of the scrollpane
   * in the horizontal/vertical dimension.
   *
   * Note that when both scrollbars are not shown, the radar view
   * window is hidden and the radar view checkbox is also not shown.
   * If a developer wants the radar view to display when both scrollbars
   * are not shown, s/he should detach the radar view and embed
   * it in another AWT container.
   */
  public static final int SCROLLBARS_AS_NEEDED = java.awt.ScrollPane.SCROLLBARS_AS_NEEDED;

  /**
   * Specifies that horizontal/vertical scrollbars should always be
   * shown regardless of the respective sizes of the scrollpane and child.
   */
  public static final int SCROLLBARS_ALWAYS = java.awt.ScrollPane.SCROLLBARS_ALWAYS;

  /**
   * Specifies that horizontal/vertical scrollbars should never be shown
   * regardless of the respective sizes of the scrollpane and child.
   * The radar view checkbox will not be displayed.
   *
   * This mode is more useful here in RadarPane than in ScrollPane, 
   * since users may still navigate the viewport using the radar view.
   */
  public static final int SCROLLBARS_NEVER = java.awt.ScrollPane.SCROLLBARS_NEVER;

  /**
    Specifies under what conditions to display the scrollbars.  
    One of: SCROLLBARS_AS_NEEDED, SCROLLBARS_ALWAYS, SCROLLBARS_NEVER.  
    The default is SCROLLBARS_AS_NEEDED.
   */
  protected int scrollbarDisplayPolicy;

  /**
    The vertical Adjustable (scrollbar)
   */
  protected Scrollbar vAdjustable = null;

  /**
    The horizontal Adjustable (scrollbar)
   */
  protected Scrollbar hAdjustable = null;

  /**
    The panel which embeds the client of this RadarPane.
   */
  protected RadarPaneClientPanel clientPanel = new RadarPaneClientPanel();

  /**
    The selector that toggles the view of the radarView.
   */
  protected Checkbox radarToggle = null;

  /**
    The radar view component.
   */
  protected RadarView radarView = null;

  /**
    The window that displays the radar view.
   */
  protected ViewWindow radarViewWindow = null;

  /**
    Indicates wether the radar view is attached to the radarViewWindow.
    true if  the radar view is attached to the radarViewWindow.
    false otherwise.
   */
  protected boolean radarViewAttached = true;

  /**
   * Create a new RadarPane container with a scrollbar display policy of
   * "as needed". 
   */
  public RadarPane() {
    this(SCROLLBARS_AS_NEEDED);
  }

  /**
   * Create a new RadarPane container.
   * @param scrollbarDisplayPolicy  policy for when scrollbars should be shown
   */
  public RadarPane(int scrollbarDisplayPolicy) {
    copyright();
    switch (scrollbarDisplayPolicy) {
    case ScrollPane.SCROLLBARS_NEVER:
    case ScrollPane.SCROLLBARS_AS_NEEDED:
    case ScrollPane.SCROLLBARS_ALWAYS:
      this.scrollbarDisplayPolicy = scrollbarDisplayPolicy;
      break;
    default:
      throw new IllegalArgumentException("illegal scrollbar display policy");
    }
    
    setLayout(new RadarPaneLayout(2,2));
    add(clientPanel, "Client");
    
    if (scrollbarDisplayPolicy != ScrollPane.SCROLLBARS_NEVER) {
      vAdjustable = new Scrollbar(Adjustable.VERTICAL);
      hAdjustable = new Scrollbar(Adjustable.HORIZONTAL);
      add(vAdjustable, "Vertical");
      add(hAdjustable, "Horizontal");
    }
    radarView = new RadarView(clientPanel, this);
    clientPanel.setRadarView(radarView);
    //    validate();  // Remove this if not needed
  }


  /**
   * Create a new RadarPane container from an existing ScrollPane.
   * @param scrollPane  the ScrollPane from which to construct this 
   *    RadarPane.  The RadarPane will have the same state as the
   *    scrollPane.
   */
  public RadarPane(java.awt.ScrollPane scrollPane) {
	copyright();
    scrollbarDisplayPolicy = SCROLLBARS_ALWAYS;
    //    scrollbarDisplayPolicy = scrollPane.getScrollbarDisplayPolicy();

    setLayout(new RadarPaneLayout(2,2));
    add(clientPanel, "Client");
    
    if (scrollbarDisplayPolicy != SCROLLBARS_NEVER) {
      vAdjustable = new Scrollbar(Adjustable.VERTICAL);
      hAdjustable = new Scrollbar(Adjustable.HORIZONTAL);
      copyAdjustable(scrollPane.getVAdjustable(), vAdjustable);
      copyAdjustable(scrollPane.getHAdjustable(), hAdjustable);

      add(vAdjustable, "Vertical");
      add(hAdjustable, "Horizontal");
    }
   
    radarView = new RadarView(clientPanel, this);

    // get the viewable component
    try {
      Component clientComp = scrollPane.getComponent(0);
      Rectangle clientBounds = clientComp.getBounds();
      debug("Client Comp = "+clientComp);
      add(clientComp);
    } catch (Exception ex) {
      System.out.println("Exception in scrollPane.getComponent(0)"+ex);
      ex.printStackTrace();
    }

    validate();
    Rectangle spBounds = scrollPane.getBounds();
    setBounds(spBounds.x, spBounds.y, spBounds.width, spBounds.height);
    debug("scrollPane Bounds = "+spBounds);
    // setScrollPosition(-clientBounds.x, -clientBounds.y);
  }

  private void copyAdjustable(Adjustable source, Adjustable dest) {
    dest.setBlockIncrement(  source.getBlockIncrement());
    dest.setUnitIncrement(   source.getUnitIncrement());
    dest.setMaximum(         source.getMaximum());
    dest.setMinimum(         source.getMinimum());
    dest.setVisibleAmount(   source.getVisibleAmount());
    dest.setValue(           source.getValue());
  }

  /**
   * Adds the specified component to this container at the specified
   * index.  Also notifies the layout manager to add the component to
   * this container's layout using the specified constraints object.
   * <p>
   * This is overriden for RadarPane so that components are added only
   * to the viewport region of the RadarPane.
   * <p>
   * This is the method to override if you want to track every add
   * request to a container.  An overriding method should usually
   * include a call to super.addImpl(comp, constraints, index).
   * @param comp the component to be added
   * @param constraints an object expressing layout contraints for this
   * component
   * @param index the position in the container's list at which to
   * insert the component.  -1 means insert at the end.
   * @see #remove
   * @see LayoutManager
   */
  protected void addImpl(Component comp, Object constraints, int index) {
    debug("addImpl Comp="+comp+", constraints="+constraints+", index="+index);
    if ( ("Vertical".equals(constraints)) || 
	 ("Horizontal".equals(constraints)) || 
	 ("Client".equals(constraints)) ||
	 ("RadarToggle".equals(constraints))) {
      super.addImpl(comp, constraints, index);
    } else {
      clientPanel.addImpl(comp, constraints, index);
    }
  }

  /** 
   * Adds the specified component to this scroll pane container.
   * If the scroll pane has an existing child component, that
   * component is removed and the new one is added.  
   * @param comp the component to be added 
   */
  public Component add(Component comp) {
    /*
     *  This method is overriden only so that the above
     *  comments will be displayed in the javadoc documentation
     */
    debug("add( "+comp+" )");
    return super.add(comp);
  }

  /**
   *  Gets the radar view Component.
   *  The radar view component gives a scaled down 
   *  full view of the client component of this RadarPane
   *  and provides a navigation mechanism in addition to the
   *  scroll bars of this RadarPane.
   *
   *  Use this, if you want the radar view window and the 
   *  radar view checkbox to no longer exist.  
   *  Otherwise, use getRadarView() .
   *
   * @see #getRadarView
   * @return the radar view Component
   */
  public Component detachRadarView() {
    if (radarViewWindow != null) {
      radarViewWindow.setVisible(false);
radarViewWindow.remove(getRadarView()); //PLI
      radarViewWindow = null;
    }
    if (radarToggle != null) {
      remove(radarToggle);
      radarToggle = null;
    }
    radarViewAttached = false;
    return getRadarView();
  }
   
  /**
   *  Gets the radar view Component.
   *  The radar view component gives a scaled down 
   *  full view of the client component of this RadarPane
   *  and provides a navigation mechanism in addition to the
   *  scroll bars of this RadarPane.
   *
   *  Use this, if you want the radar view window and the radar
   *  radar view checkbox to continue to exist.  
   *  Otherwise, use detachRadarView() .
   *
   * @see #detachRadarView
   * @return the radar view Component
   */
  public Component getRadarView() {
    return radarView;
  }
   
  /**
   *  Displays the radar view window and the
   *  radar view checkbox. 
   *
   * @see #detachRadarView
   */
  public void attachRadarView() {
    if (radarView != null )
      radarView.setLocalUser();
    if (radarViewWindow == null) {
      Component comp = this;
      for ( ; !(comp instanceof Frame); comp = comp.getParent());
      
      radarViewWindow = new ViewWindow((Frame)comp, "Radar View Window", 
                                       radarView, this);
    }
    radarViewWindow.setVisible(true);
    radarView.setVisible(true);
      
    if ((radarToggle == null) && 
	(scrollbarDisplayPolicy != ScrollPane.SCROLLBARS_NEVER)) {
      radarToggle = new Checkbox();
      radarToggle.setState(true);
      add(radarToggle, "RadarToggle");   
    }
    radarViewAttached = true;
  }
   
  /** 
   * Sets the layout manager for this container.  This method is
   * overridden to prevent the layout mgr from being set.
   * @param mgr the specified layout manager
   */
  public final void setLayout(LayoutManager mgr) {
    if (mgr instanceof RadarPaneLayout)
      super.setLayout(mgr);
  }

  /**
   * Returns the display policy for the scrollbars.
   * @return the display policy for the scrollbars
   */
  public int getScrollbarDisplayPolicy() {
    return scrollbarDisplayPolicy;
  }

  /**
   * Returns the current size of the scroll pane's view port.
   * @return the size of the view port in pixels
   */
  public Dimension getViewportSize() {
    return  clientPanel.getSize();
  }
  /**
   * Returns the height that would be occupied by a horizontal
   * scrollbar, which is independent of whether it is currently
   * displayed by the scroll pane or not.
   * @return the height of a horizontal scrollbar in pixels
   */
  public int getHScrollbarHeight() {
    if (hAdjustable != null)
      return hAdjustable.getSize().height;
    else 
      return getSize().height;
  }

  /**
   * Returns the width that would be occupied by a vertical
   * scrollbar, which is independent of whether it is currently
   * displayed by the scroll pane or not.
   * @return the width of a vertical scrollbar in pixels
   */
  public int getVScrollbarWidth() {
    if (vAdjustable != null)
      return vAdjustable.getSize().width;
    else 
      return getSize().width;
  }

  /**
   * Returns the Adjustable object which represents the state of
   * the vertical scrollbar. If the scrollbar display policy is "never",
   * this method returns null.
   */
  public Adjustable getVAdjustable() {
    return vAdjustable;
  }

  /**
   * Returns the Adjustable object which represents the state of
   * the horizontal scrollbar.  If the scrollbar display policy is "never",
   * this method returns null.
   */
  public Adjustable getHAdjustable() {
    return hAdjustable;
  }

  /**
   * Scrolls to the specified position within the child component.
   * A call to this method is only valid if the scroll pane contains
   * a child and the specified position is within legal scrolling bounds
   * of the child.  Legal bounds are defined to be the rectangle: 
   * x = 0, y = 0, width = (child width - view port width),
   * height = (child height - view port height).
   * This is a convenience method which interfaces with the Adjustable
   * objects which respresent the state of the scrollbars.
   * @param x the x position to scroll to
   * @param y the y position to scroll to
   * @exception IllegalArgumentException if specified coordinates are
   * not within the legal scrolling bounds of the child component.
   */
  public void setScrollPosition(int x, int y) {
    if (scrollbarDisplayPolicy != ScrollPane.SCROLLBARS_NEVER) {
      hAdjustable.setValue(x);
      vAdjustable.setValue(y);
      scrollMoveClient();
    } else {
      moveClient(-x, -y);
    }
    debug("x = "+x+" y = "+y);
    debug("hAdjustable = "+hAdjustable+", vAdjustable = "+vAdjustable);
    if (radarView != null)
      radarView.setLocalUser();
  }

  /**
   * Scrolls to the specified position within the child component.
   * A call to this method is only valid if the scroll pane contains
   * a child and the specified position is within legal scrolling bounds
   * of the child.  Legal bounds are defined to be the rectangle: 
   * x = 0, y = 0, width = (child width - view port width),
   * height = (child height - view port height).
   * This is a convenience method which interfaces with the Adjustable
   * objects which respresent the state of the scrollbars.
   * @param p the Point representing the position to scroll to
   * @exception IllegalArgumentException if specified coordinates are
   * not within the legal scrolling bounds of the child component.
   */
  public void setScrollPosition(Point p) {
    setScrollPosition(p.x, p.y);
  }

  /**
   * Returns the current x,y position within the child which is displayed 
   * at the 0,0 location of the scrolled panel's view port.
   * This is a convenience method which interfaces with the adjustable
   * objects which respresent the state of the scrollbars.
   * @return the coordinate position for the current scroll position
   */
  public Point getScrollPosition() {
    Component c = clientPanel.getComponent(0);
    if (c != null) {
      Point p = c.getLocation();
      return new Point(-p.x, -p.y);
    } else
      return new Point(0, 0);
  }

  /**
    Handles scroll bar and radar view events that move the embedded 
    client component.
   * @return  super.handleEvent(evt);
   */
  public boolean handleEvent(Event evt) {
    switch (evt.id) {
    case Event.SCROLL_LINE_UP:
    case Event.SCROLL_LINE_DOWN:
    case Event.SCROLL_PAGE_UP:
    case Event.SCROLL_PAGE_DOWN:
    case Event.SCROLL_ABSOLUTE:
      scrollMoveClient();
      break;
    case Event.ACTION_EVENT:
      if (evt.target == radarToggle) {
	radarViewWindow.setVisible(radarToggle.getState());
      }
      break;
    }
    return super.handleEvent(evt);
  }

  /**
    Hides the radar view window.
   */
  public void hideViewWindow() {
    if (radarToggle != null)
      radarToggle.setState(false);
    if (radarViewWindow != null)
      radarViewWindow.setVisible(false);
    radarView.setVisible(false);
    //  radarView = null;
  }

  /**
   * Moves the embedded client component according to the current scroll bar
   * position values. Used by setScrollPosition.
   * @see #setScrollPosition
   * @see #moveClient
   */
  private void scrollMoveClient() {
    moveClient(-(hAdjustable.getValue()),
	       -(vAdjustable.getValue()));
  }
  
  /**
   * Moves the embedded client component.
   * @see #setScrollPosition
   * @see #scrollMoveClient
  */
  private void moveClient(int x, int y) {
    clientPanel.getComponent(0).setLocation(x, y);
    clientPanel.repaint();
    if (radarView != null) {
      radarView.repaint();
      radarView.setLocalUser();
      debug("moveClient("+x+", "+y+")");
    }
  }

  /**
    Calls paint(g) without first clearing the component background.
   */
  public void update(Graphics g) {
    paint(g);
  }

  /**
    Draws the RadarPane and calls repaint on the radar view component.
   */
  public void paint(Graphics g) {
    super.paint(g);

    if (radarView != null) {
      // update the radar view
      radarView.repaint();
    }
  }

  /**
    Overriden to also invalidate the radar view component.
   */
  public void invalidate() {
    debug("invalidate()");
    if (radarView != null) {
      radarView.invalidate();
    }
    super.invalidate();
    if (radarView != null) {
      radarView.setRadarPaneChange(true);
    }
  }

  /**
    Overriden to also validate the radar view component and its parent.
   */
  public void validate() {
    debug("validate()");
    if (radarView != null) {
      radarView.validate();
      if (radarView.getParent() != null)
      	radarView.getParent().validate();
    }
    super.validate();
    if (radarView != null) {
      radarView.setRadarPaneChange(true);
    }
  }
  
  /**
    Overriden to attach the radar view component when this 
    RadarPane is added to something.
   */
  public void addNotify() {
    debug("addNotify");
    if (radarViewAttached)
      attachRadarView();
    super.addNotify();
  }
  
  /**
    Overriden to detach the radar view component when this 
    RadarPane is removed from something.
   */
  public void removeNotify() {
    Component comp = detachRadarView();
  }

  /**
    Sets minimum size to Dimension(200, 200), unless the size is 
    explicitly set to be less.
    @see #getPreferredSize
   */
  public Dimension getMinimumSize() {
    debug("getMinimumSize()");
    Dimension dim = getSize();
    return new Dimension(Math.min(dim.width, 200),
			 Math.min(dim.height, 200));
  }

  /** 
    Sets the preferred size to call getMinimumSize
    @see #getMinimumSize
    */
  public Dimension getPreferredSize() {
    debug("getPreferredSize()");
    Dimension dim = getSize();
    Dimension min = getMinimumSize();
    return new Dimension(Math.max(dim.width, min.width),
			 Math.max(dim.height, min.height));
  }

    /**
     * Sets an object to be used for scaling the contents of the
     * RadarPane. If no scaler is set, a check is made to see if 
     * the component within the RadarPane implements Scalable. If it
     * does not, then a snapshot of the contained component is taken
     * and scaled.
     * @param scaler An object which knows how to scale the component contained within the RadarPane
     * @see #getScaler
     */
    public void setScaler(Scalable scaler) {
	radarView.setScaler(scaler);
    }

    /**
     * Returns the object which is responsible for scaling the component
     * contained within the RadarPane, if such an object has been registered
     * with the setScaler method.
     * @see #setScaler
     */
    public Scalable getScaler() {
        return radarView.getScaler();
    }


    /** 
     * Reshapes the Component to the specified bounding box.
     * Overriden to notify the radar view that the view port has changed.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the component
     * @param height the height of the component
     * @see #getBounds
     * @see #setLocation
     * @see #setSize
     */
  public void setBounds(int x, int y, int width, int height) {
    debug("setBounds("+x+", "+y+", "+width+", "+height+")");
    super.setBounds(x, y, width, height);
    if (radarView != null) {
      radarView.setRadarPaneChange(true);
    }
    debug("bounds="+getBounds());
  }


    public String paramString() {
	String sdpStr;
	switch (scrollbarDisplayPolicy) {
	    case SCROLLBARS_AS_NEEDED:
		sdpStr = "as-needed";
		break;
	    case SCROLLBARS_ALWAYS:
		sdpStr = "always";
		break;
	    case SCROLLBARS_NEVER:
		sdpStr = "never";
		break;
	    default:
		sdpStr = "invalid display policy";
	}
	Point p = (clientPanel.getComponent(0) != null)? 
		getScrollPosition() : 
		new Point(0,0);
	Insets i = getInsets();		
	return super.paramString()+",ScrollPosition=("+p.x+","+p.y+")"+
	    ",Insets=("+i.top+","+i.left+","+i.bottom+","+i.right+")"+
	    ",ScrollbarDisplayPolicy="+sdpStr;
    }
  /**
   * Sets or clears the clientChanged flag.
   * @param value true causes radar view to get a new image of the embedded
   *        client.  false clears the flag; radar view does nothing, unless 
   *        flag was set previously and it has not yet obtained the new 
   *        image.  In which case, it will not get the new image.
   */      
  public void setClientChanged(boolean value) {
    radarView.setClientChanged(value);
  }  
  
  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}


