/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
package vt.cs.widgets;

import java.awt.*;

/**
 *
 * @version 	$Revision: 1.8 $
 * @author 	James Begole <a href="mailto:begolej@vt.edu>begolej@vt.edu</a>
 */


public class RadarPanePeer
extends java.awt.Panel 
implements java.io.Serializable, ViewWindowParent {

  private static String revisionString = "$Revision: 1.8 $";
  private void copyright() {
    System.out.println(getClass().getName()+": "+revisionString+"\n"+
	    "  (c) copyright 1997, James Begole, All rights reseverd.\n"+
	    "  Permission is granted to use this class unmodified.");
  }

  /**
    Specifies under what conditions to display the scrollbars.  
    One of: SCROLLBARS_AS_NEEDED, SCROLLBARS_ALWAYS, SCROLLBARS_NEVER.  
    The default is SCROLLBARS_AS_NEEDED.
   */
  private int scrollbarDisplayPolicy;  
  
  /**
    The vertical Adjustable (scrollbar)
   */
  private Scrollbar vAdjustable = null;

  /**
    The horizontal Adjustable (scrollbar)
   */
  private Scrollbar hAdjustable = null;

  /**
    The panel which embeds the client of this RadarPane.
   */
  private RadarPaneClientPanel clientPanel = new RadarPaneClientPanel();

  /**
    The selector that toggles the view of the radarView.
   */
  private Checkbox radarToggle = null;

  /**
    The radar view component.
   */
  private RadarPeerView radarView = null;

  /**
    The window that displays the radar view.
   */
  private ViewWindow radarViewWindow = null;

  /**
    Indicates wether the radar view is attached to the radarViewWindow.
    true if  the radar view is attached to the radarViewWindow.
    false otherwise.
   */
  private boolean radarViewAttached = true;

  /**
   * Create a new RadarPane container with a scrollbar display policy of
   * "as needed". 
   */
  public RadarPanePeer() {
    this(RadarPaneS.SCROLLBARS_AS_NEEDED);
  }

  /**
   * Create a new RadarPanePeer container.
   * @param scrollbarDisplayPolicy  policy for when scrollbars should be shown
   */
  public RadarPanePeer(int scrollbarDisplayPolicy) {
    copyright();
    switch (scrollbarDisplayPolicy) {
    case RadarPaneS.SCROLLBARS_NEVER:
    case RadarPaneS.SCROLLBARS_AS_NEEDED:
    case RadarPaneS.SCROLLBARS_ALWAYS:
      this.scrollbarDisplayPolicy = scrollbarDisplayPolicy;
      break;
    default:
      throw new IllegalArgumentException("illegal scrollbar display policy");
    }
    
    setLayout(new RadarPanePeerLayout(2,2));
    add(clientPanel, "Client");
    
    if (scrollbarDisplayPolicy != RadarPaneS.SCROLLBARS_NEVER) {
      vAdjustable = new Scrollbar(Adjustable.VERTICAL);
      hAdjustable = new Scrollbar(Adjustable.HORIZONTAL);
      add(vAdjustable, "Vertical");
      add(hAdjustable, "Horizontal");
    }
    radarView = new RadarPeerView(clientPanel, this);
    validate();  // Remove this if not needed
  }


  /**
   * Create a new RadarPane container from an existing ScrollPane.
   * @param scrollPane  the ScrollPane from which to construct this 
   *    RadarPane.  The RadarPane will have the same state as the
   *    scrollPane.
   */
  public RadarPanePeer(java.awt.ScrollPane scrollPane) {
	copyright();
    // scrollbarDisplayPolicy = RadarPaneS.SCROLLBARS_ALWAYS;
    scrollbarDisplayPolicy = scrollPane.getScrollbarDisplayPolicy();

    setLayout(new RadarPanePeerLayout(2,2));
    add(clientPanel, "Client");
    
    if (scrollbarDisplayPolicy != RadarPaneS.SCROLLBARS_NEVER) {
      vAdjustable = new Scrollbar(Adjustable.VERTICAL);
      hAdjustable = new Scrollbar(Adjustable.HORIZONTAL);
      copyAdjustable(scrollPane.getVAdjustable(), vAdjustable);
      copyAdjustable(scrollPane.getHAdjustable(), hAdjustable);

      add(vAdjustable, "Vertical");
      add(hAdjustable, "Horizontal");
    }
   
    radarView = new RadarPeerView(clientPanel, this);

    // get the viewable component
    try {
      Component clientComp = scrollPane.getComponent(0);
      Rectangle clientBounds = clientComp.getBounds();
      debug("Client Comp = "+clientComp);
      add(clientComp);
    } catch (Exception ex) {
      System.out.println("Exception in scrollPane.getComponent(0)"+ex);
      ex.printStackTrace();
    }

    validate();
    Rectangle spBounds = scrollPane.getBounds();
    setBounds(spBounds.x, spBounds.y, spBounds.width, spBounds.height);
    debug("scrollPane Bounds = "+spBounds);
    // setScrollPosition(-clientBounds.x, -clientBounds.y);
  }

  private void copyAdjustable(Adjustable source, Adjustable dest) {
    dest.setBlockIncrement(  source.getBlockIncrement());
    dest.setUnitIncrement(   source.getUnitIncrement());
    dest.setMaximum(         source.getMaximum());
    dest.setMinimum(         source.getMinimum());
    dest.setVisibleAmount(   source.getVisibleAmount());
    dest.setValue(           source.getValue());
  }

  /**
   * Adds the specified component to this container at the specified
   * index.  Also notifies the layout manager to add the component to
   * this container's layout using the specified constraints object.
   * <p>
   * This is overriden for RadarPane so that components are added only
   * to the viewport region of the RadarPane.
   * <p>
   * This is the method to override if you want to track every add
   * request to a container.  An overriding method should usually
   * include a call to super.addImpl(comp, constraints, index).
   * @param comp the component to be added
   * @param constraints an object expressing layout contraints for this
   * component
   * @param index the position in the container's list at which to
   * insert the component.  -1 means insert at the end.
   * @see #remove
   * @see LayoutManager
   */
  protected void addImpl(Component comp, Object constraints, int index) {
    debug("addImpl Comp="+comp+", constraints="+constraints+", index="+index);
    if ( ("Vertical".equals(constraints)) || 
	 ("Horizontal".equals(constraints)) || 
	 ("Client".equals(constraints)) ||
	 ("RadarToggle".equals(constraints))) {
      super.addImpl(comp, constraints, index);
    } else {
      clientPanel.addImpl(comp, constraints, index);
    }
  }

  /** 
   * Adds the specified component to this scroll pane container.
   * If the scroll pane has an existing child component, that
   * component is removed and the new one is added.  
   * @param comp the component to be added 
   */
  public Component add(Component comp) {
    /*
     *  This method is overriden only so that the above
     *  comments will be displayed in the javadoc documentation
     */
    debug("add( "+comp+" )");
    return super.add(comp);
  }

  /**
   *  Gets the radar view Component.
   *  The radar view component gives a scaled down 
   *  full view of the client component of this RadarPane
   *  and provides a navigation mechanism in addition to the
   *  scroll bars of this RadarPane.
   *
   *  Use this, if you want the radar view window and the 
   *  radar view checkbox to no longer exist.  
   *  Otherwise, use getRadarView() .
   *
   * @see #getRadarView
   * @return the radar view Component
   */
  public Component detachRadarView() {
    radarViewWindow.setVisible(false);
    radarViewWindow = null;
    if (radarToggle != null) {
      remove(radarToggle);
      radarToggle = null;
    }
    radarViewAttached = false;
    return getRadarView();
  }
   
  /**
   *  Gets the radar view Component.
   *  The radar view component gives a scaled down 
   *  full view of the client component of this RadarPane
   *  and provides a navigation mechanism in addition to the
   *  scroll bars of this RadarPane.
   *
   *  Use this, if you want the radar view window and the radar
   *  radar view checkbox to continue to exist.  
   *  Otherwise, use detachRadarView() .
   *
   * @see #detachRadarView
   * @return the radar view Component
   */
  public Component getRadarView() {
    return radarView;
  }

  public Container parent = null;
  
  public Container getParent() {
	return parent;
  }
   
  /**
   *  Displays the radar view window and the
   *  radar view checkbox. 
   *
   * @see #detachRadarView
   */
  public void attachRadarView() {
    if (radarViewWindow == null) {
      Component comp = this;
      for ( ; !(comp instanceof Frame); comp = comp.getParent());
      
      radarViewWindow = new ViewWindow((Frame)comp, "Radar View Window", 
                                       radarView, this);
    }
    radarViewWindow.setVisible(true);
      
    if ((radarToggle == null) && 
	(scrollbarDisplayPolicy != ScrollPane.SCROLLBARS_NEVER)) {
      radarToggle = new Checkbox();
      radarToggle.setState(true);
      add(radarToggle, "RadarToggle");   
    }
    radarViewAttached = true;
  }
   
  /** 
   * Sets the layout manager for this container.  This method is
   * overridden to prevent the layout mgr from being set.
   * @param mgr the specified layout manager
   */
  public final void setLayout(LayoutManager mgr) {
    if (mgr instanceof RadarPanePeerLayout)
      super.setLayout(mgr);
  }

  /**
   * Returns the display policy for the scrollbars.
   * @return the display policy for the scrollbars
   */
  public int getScrollbarDisplayPolicy() {
    return scrollbarDisplayPolicy;
  }

  /**
   * Returns the current size of the scroll pane's view port.
   * @return the size of the view port in pixels
   */
  public Dimension getViewportSize() {
    return  clientPanel.size();
  }
  /**
   * Returns the height that would be occupied by a horizontal
   * scrollbar, which is independent of whether it is currently
   * displayed by the scroll pane or not.
   * @return the height of a horizontal scrollbar in pixels
   */
  public int getHScrollbarHeight() {
    if (hAdjustable != null)
      return hAdjustable.size().height;
    else 
      return size().height;
  }

  /**
   * Returns the width that would be occupied by a vertical
   * scrollbar, which is independent of whether it is currently
   * displayed by the scroll pane or not.
   * @return the width of a vertical scrollbar in pixels
   */
  public int getVScrollbarWidth() {
    if (vAdjustable != null)
      return vAdjustable.size().width;
    else 
      return size().width;
  }

  /**
   * Returns the Adjustable object which represents the state of
   * the vertical scrollbar. If the scrollbar display policy is "never",
   * this method returns null.
   */
  public Adjustable getVAdjustable() {
    return vAdjustable;
  }

  /**
   * Returns the Adjustable object which represents the state of
   * the horizontal scrollbar.  If the scrollbar display policy is "never",
   * this method returns null.
   */
  public Adjustable getHAdjustable() {
    return hAdjustable;
  }

  /**
   * Scrolls to the specified position within the child component.
   * A call to this method is only valid if the scroll pane contains
   * a child and the specified position is within legal scrolling bounds
   * of the child.  Legal bounds are defined to be the rectangle: 
   * x = 0, y = 0, width = (child width - view port width),
   * height = (child height - view port height).
   * This is a convenience method which interfaces with the Adjustable
   * objects which respresent the state of the scrollbars.
   * @param x the x position to scroll to
   * @param y the y position to scroll to
   * @exception IllegalArgumentException if specified coordinates are
   * not within the legal scrolling bounds of the child component.
   */
  public void setScrollPosition(int x, int y) {
    if (scrollbarDisplayPolicy != ScrollPane.SCROLLBARS_NEVER) {
      hAdjustable.setValue(x);
      vAdjustable.setValue(y);
      scrollMoveClient();
    } else {
      moveClient(-x, -y);
    }
    debug("x = "+x+" y = "+y);
    debug("hAdjustable = "+hAdjustable+", vAdjustable = "+vAdjustable);
  }

  /**
   * Scrolls to the specified position within the child component.
   * A call to this method is only valid if the scroll pane contains
   * a child and the specified position is within legal scrolling bounds
   * of the child.  Legal bounds are defined to be the rectangle: 
   * x = 0, y = 0, width = (child width - view port width),
   * height = (child height - view port height).
   * This is a convenience method which interfaces with the Adjustable
   * objects which respresent the state of the scrollbars.
   * @param p the Point representing the position to scroll to
   * @exception IllegalArgumentException if specified coordinates are
   * not within the legal scrolling bounds of the child component.
   */
  public void setScrollPosition(Point p) {
    setScrollPosition(p.x, p.y);
  }

  /**
   * Returns the current x,y position within the child which is displayed 
   * at the 0,0 location of the scrolled panel's view port.
   * This is a convenience method which interfaces with the adjustable
   * objects which respresent the state of the scrollbars.
   * @return the coordinate position for the current scroll position
   */
  public Point getScrollPosition() {
    Component c = clientPanel.getComponent(0);
    if (c != null) {
      Point p = c.getLocation();
      return new Point(-p.x, -p.y);
    } else
      return new Point(0, 0);
  }

  /**
    Handles scroll bar and radar view events that move the embedded 
    client component.
   * @return  super.handleEvent(evt);
   */
  public boolean handleEvent(Event evt) {
    switch (evt.id) {
    case Event.SCROLL_LINE_UP:
    case Event.SCROLL_LINE_DOWN:
    case Event.SCROLL_PAGE_UP:
    case Event.SCROLL_PAGE_DOWN:
    case Event.SCROLL_ABSOLUTE:
      scrollMoveClient();
      break;
    case Event.ACTION_EVENT:
      if (evt.target == radarToggle) {
	radarViewWindow.setVisible(radarToggle.getState());
      }
      break;
    }
    return super.handleEvent(evt);
  }

  /**
    Hides the radar view window.
   */
  public void hideViewWindow() {
    if (radarToggle != null)
      radarToggle.setState(false);
    if (radarViewWindow != null)
      radarViewWindow.setVisible(false);
  }

  /**
   * Moves the embedded client component according to the current scroll bar
   * position values. Used by setScrollPosition.
   * @see #setScrollPosition
   * @see #moveClient
   */
  private void scrollMoveClient() {
    moveClient(-(hAdjustable.getValue()),
	       -(vAdjustable.getValue()));
  }
  
  /**
   * Moves the embedded client component.
   * @see #setScrollPosition
   * @see #scrollMoveClient
  */
  private void moveClient(int x, int y) {
    clientPanel.getComponent(0).setLocation(x, y);
    clientPanel.repaint();
    if (radarView != null) {
      radarView.repaint();
      debug("moveClient("+x+", "+y+")");
    }
  }

  /**
    Calls paint(g) without first clearing the component background.
   */
  public void update(Graphics g) {
    paint(g);
  }

  /**
    Draws the RadarPane and calls repaint on the radar view component.
   */
  public void paint(Graphics g) {
    super.paint(g);

    if (radarView != null) {
      // update the radar view
      debug("paint()");
      radarView.repaint();
    }
  }

  /**
    Overriden to also invalidate the radar view component.
   */
  public void invalidate() {
    debug("invalidate()");
    if (radarView != null) {
      radarView.invalidate();
    }
    super.invalidate();
  }

  /**
    Overriden to also validate the radar view component and its parent.
   */
  public void validate() {
    debug("validate()");
    if (radarView != null) {
      radarView.validate();
      if (radarView.getParent() != null)
      	radarView.getParent().validate();
    }
    super.validate();
  }
  
  /**
    Overriden to attach the radar view component when this 
    RadarPane is added to something.
   */
  public void addNotify() {
    debug("addNotify");
    attachRadarView();
    super.addNotify();
  }
  
  /**
    Overriden to detach the radar view component when this 
    RadarPane is removed from something.
   */
  public void removeNotify() {
    Component comp = detachRadarView();
  }

  /**
    Sets minimum size to Dimension(200, 200), unless the size is 
    explicitly set to be less.
    @see #getPreferredSize
   */
  public Dimension getMinimumSize() {
    debug("getMinimumSize()");
    Dimension dim = size();
    return new Dimension(Math.min(dim.width, 200),
			 Math.min(dim.height, 200));
  }

  /** 
    Sets the preferred size to call getMinimumSize
    @see #getMinimumSize
    */
  public Dimension getPreferredSize() {
    debug("getPreferredSize()");
    Dimension dim = size();
    Dimension min = getMinimumSize();
    return new Dimension(Math.max(dim.width, min.width),
			 Math.max(dim.height, min.height));
  }
  
  
  public void processEvent(AWTEvent e) {
    super.processEvent(e);
  }

    public String paramString() {
	String sdpStr;
	switch (scrollbarDisplayPolicy) {
	    case RadarPaneS.SCROLLBARS_AS_NEEDED:
		sdpStr = "as-needed";
		break;
	    case RadarPaneS.SCROLLBARS_ALWAYS:
		sdpStr = "always";
		break;
	    case RadarPaneS.SCROLLBARS_NEVER:
		sdpStr = "never";
		break;
	    default:
		sdpStr = "invalid display policy";
	}
	Point p = (clientPanel.getComponent(0) != null)? 
		getScrollPosition() : 
		new Point(0,0);
	Insets i = getInsets();		
	return super.paramString()+",ScrollPosition=("+p.x+","+p.y+")"+
	    ",Insets=("+i.top+","+i.left+","+i.bottom+","+i.right+")"+
	    ",ScrollbarDisplayPolicy="+sdpStr;
    }
  
  
  
  
  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }

}


/****************************************
RadarView is the component that displays the radar view.
****************************************/

class RadarPeerView extends Panel{
  public static final int defaultScaleFactor = 3;

  private Image bgImage = null;
  private Image scaledBgImage = null;
  private RadarPaneClientPanel clientPanel = null;
  private RadarPanePeer scroller = null;

  RadarPeerView(RadarPaneClientPanel clientPanel, RadarPanePeer scroller) {
    this.clientPanel = clientPanel;
    this.scroller = scroller;
  }

  public void update(Graphics g) {
    paint(g);
  }
  
  Dimension clientSize = null;
  Dimension size = null;
  int imgX, imgY, imgWidth, imgHeight;
  Dimension imgSize = null;
  Insets insets = null;
  Graphics scaledBgGraphics = null;

  public void paint(Graphics g) {
    debug("paint()");
    if (!clientPanel.getComponent(0).isValid())
      return;
      
    Dimension checkClientSize = clientPanel.getComponent(0).size();
    if (checkClientSize.equals(new Dimension(0, 0)))
      return;
      
    if ((bgImage == null) || 
	(clientSize == null) ||
	(!(clientSize.equals(checkClientSize )))) {
      // Client has changed size, get a new bgImage
      clientSize = checkClientSize;
      bgImage = createImage(clientSize.width, clientSize.height);
      Graphics offgraphics = bgImage.getGraphics();
      clientPanel.getComponent(0).printAll(offgraphics);
      scaledBgImage = null;
    }

    Dimension checkSize = size();
    if ((size == null) ||
    	(scaledBgImage == null) ||
	(!size.equals(checkSize))) {
      // The radar view has changed size
      size = checkSize;
      insets = getInsets();
      imgX = insets.left;
      imgY = insets.top;
      imgWidth = size.width-(insets.left+insets.right);
      imgHeight = size.height-(insets.top+insets.bottom);
      imgSize = new Dimension(imgWidth, imgHeight);
      scaledBgImage =  createImage(imgWidth, imgHeight);
      scaledBgGraphics = scaledBgImage.getGraphics();
      scaledBgGraphics.drawImage(bgImage, 0, 0, imgWidth, imgHeight,
				 this);
    }
    //  drawUserRects(scaledBgGraphics);
    g.drawImage(scaledBgImage, imgX, imgY, this);
    drawUserRects(g);
  } 


  Rectangle userRect = null;
  
  private void drawUserRects(Graphics g) {
    if (clientPanel == null)
      return;

    Component comp = clientPanel.getComponent(0);
    if (comp == null)
      return;

    Point clientLoc = comp.getLocation();
    Dimension visible = clientPanel.size();

    g.setXORMode(Color.black);
    g.setColor(Color.green);

    float horizScale = (float)imgSize.width/(float)clientSize.width;
    float vertScale = (float)imgSize.height/(float)clientSize.height;
    int scaledX = insets.left+Math.round(-clientLoc.x * horizScale);
    int scaledWidth = Math.round(visible.width * horizScale);
    if ((scaledWidth+scaledX)>(imgSize.width+insets.left-1)) 
      scaledWidth = imgSize.width+insets.left-1-scaledX;
    int scaledY = insets.top+Math.round(-clientLoc.y * vertScale);
    int scaledHeight =  Math.round(visible.height * vertScale);
    if ((scaledHeight+scaledY)>(imgSize.height+insets.top-1))
      scaledHeight = imgSize.height+insets.top-1-scaledY;
    
    userRect = new Rectangle(scaledX, scaledY, scaledWidth, scaledHeight);
    g.drawRect(scaledX, scaledY, scaledWidth, scaledHeight);
    g.setPaintMode();
  }

  public Dimension getMinimumSize() {
    Dimension dim = null;
    if ((clientPanel == null) || !clientPanel.isValid())
      dim = new Dimension(200, 200);
    else {
      Dimension tempSize = clientPanel.getComponent(0).size();
      dim = new Dimension(tempSize.width/defaultScaleFactor, 
			  tempSize.height/defaultScaleFactor);
    }
    debug("getMinimumSize() returning "+dim);
    return dim;
  }

  public Dimension getPreferredSize() {
    debug("getPreferredSize()");
    return getMinimumSize();
  }

  public boolean  mouseDrag(Event evt, int x, int y) {
    if ((clientSize == null) || (imgSize == null))
      return false;

    moveBox(x, y);
    return true;
  }

  public boolean  mouseUp(Event evt, int x, int y) {
    if ((clientSize == null) || (imgSize == null))
      return false;

    moveBox(x, y);
    return true;
  }

  private void moveBox(int x, int y) {  
    // Make sure x is not off screen to the left
    x = Math.max(x, 0);
    // or to the right
    if (userRect != null)
      x = Math.min(x, size().width-userRect.width-1);
    debug("size().width = "+size().width+" userRect.width = "+
	  userRect.width+" x = "+x );
      
    // and make sure y is not off the top
    y = Math.max(y, 0);
    // or off the bottom
    if (userRect != null)
      y = Math.min(y, size().height-userRect.height-1);
      
    float horizScale = (float)clientSize.width/(float)imgSize.width;
    float vertScale = (float)clientSize.height/(float)imgSize.height;

    int scaledX = Math.round((x-insets.left)*horizScale);
    int scaledY = Math.round((y-insets.top)*vertScale);
    debug("scaledX = "+scaledX+", scaledY = "+scaledY);
    scroller.setScrollPosition(scaledX, scaledY);
    repaint();
  }

  /*
    // debug overrides
    public void validate() {
    debug("validate()");
    super.validate();
    } 

    public void invalidate() {
    debug("invalidate()");
    super.invalidate();
    //   if (getParent() != null)
    //     getParent().doLayout();
      
    }
    */

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}

