package vt.cs.widgets;

import java.awt.*;

public class Scaler implements Scalable {
    private boolean doScaleSelf = true;
    private Container c = null;

    public Scaler(Container c) {
	this.c = c;
    }
    public boolean scaleSelf() {
	return doScaleSelf;
    }
    public void setScaleSelf(boolean scaleSelf) {
	doScaleSelf = scaleSelf;
    }


    public void paintScaled(Graphics g, Dimension size) {
        Rectangle scaleBounds = new Rectangle(0, 0, size.width, size.height);
        Rectangle fullBounds = c.getBounds();
 
        // A Windows AWT bug prevent clearRect from working correctly.
        // Instead use the following to blank the background.
        g.setColor(c.getBackground());
        g.fillRect(0, 0, scaleBounds.width, scaleBounds.height);
 
        Component[] comps = c.getComponents();
 
        for (int i = 0; i < comps.length; i++) 
	    paintComponent(g, comps[i], scaleBounds, fullBounds);
    }

    public void paintComponent(Graphics g, Component c, 
	Rectangle scaleBounds, Rectangle fullBounds) {
        Rectangle compBounds = scaleRectangle(c.getBounds(),
            new Dimension(scaleBounds.width, scaleBounds.height),
	    new Dimension(fullBounds.width, fullBounds.height));
/*
	g.setColor(c.getBackground());
        g.fillRect(compBounds.x, compBounds.y, compBounds.width, 
	    compBounds.height);
*/
        g.setColor(Color.black);
        g.drawRect(compBounds.x, compBounds.y, compBounds.width, 
	    compBounds.height);
    }

    public static Point scalePoint(Point toScale, Dimension relativeSize,
	Dimension fullSize) {
        if ((toScale == null) || (relativeSize == null) || (fullSize == null))
       	    return null;
 
    	float horizScale = (float)relativeSize.width/(float)fullSize.width;
    	float vertScale = (float)relativeSize.height/(float)fullSize.height;
    	int scaledX = Math.round(toScale.x * horizScale);
    	int scaledY = Math.round(toScale.y * vertScale);
    	return new Point(scaledX, scaledY);
    }

    public static Rectangle scaleRectangle(Rectangle toScale, 
	Dimension relativeSize, Dimension fullSize) {
        if ((toScale == null) || (relativeSize == null) || (fullSize == null))
      	    return null;
 
        float horizScale = (float)relativeSize.width/(float)fullSize.width;
        float vertScale = (float)relativeSize.height/(float)fullSize.height;
     	int scaledX = Math.round(toScale.x * horizScale);
    	int scaledWidth = Math.round(toScale.width * horizScale);
    	if ((scaledWidth + scaledX) > (relativeSize.width - 1))
      	    scaledWidth = relativeSize.width-1-scaledX;
    	int scaledY = Math.round(toScale.y * vertScale);
    	int scaledHeight =  Math.round(toScale.height * vertScale);
    	if ((scaledHeight + scaledY) > (relativeSize.height - 1))
      	    scaledHeight = relativeSize.height-1-scaledY;
 
     	Rectangle scaled =  new Rectangle(scaledX, scaledY, scaledWidth, 
	    scaledHeight);
    	return scaled;
   }

}
