/*
 *
 * Written By:		James "Bo" Begole
 * Date:			19 June 1996
 * Description:	Bar Separator Widget
 *
 */

/*
 *
 * Copyright (c) 1996 James "Bo" Begole, All Rights Reserved.
 *
 * 
 */
/**
 * A 3D sunken (or raised) horizontal (or vertical) bar.  Can be used
 * to separate clusters of user interface elements
 */
/** @author James "Bo" Begole
 */ 
package vt.cs.widgets;
 
import java.awt.*;

public class Separator extends Panel {
	private static final boolean debug = false;
	
	public static final int HORIZONTAL = 1;
	public static final int VERTICAL = 2;
	public static final int SUNKEN = 3;
	public static final int RAISED = 4;
	
	private int alignment = HORIZONTAL;
	private int perspective = SUNKEN;
	private int border = 1;	// size of border
	private Color highlight = null;
	private Color shadow = null;
	

	public Separator() {
	}

	public Separator(int alignment) {
		this.alignment = alignment;
	}

	public Separator(int alignment, int width) {
		border = (width/2>0)?(width/2):(1);
		this.alignment = alignment;
	}
	
	public Separator(int alignment, int width, int perspective) {
		this.alignment = alignment;
		border = (width/2>0)?(width/2):(1);
		this.perspective = perspective;
	}

	
	public void setColors(Color highlight, Color shadow) {
		this.highlight = highlight;
		this.shadow = shadow;
	}
	
	public void setAlignment(int alignment) {
		this.alignment = alignment;
	}

	public void setWidth(int width) {
		border = (width/2>0)?(width/2):(1);
	}
	
	public void setPerspective(int perspective) {
		this.perspective = perspective;
	}

	public Dimension minimumSize() {
		return new Dimension(2*border, 2*border);
	}
	
	Dimension d;
	
	public void paint(Graphics g) {
		if (debug)
			System.out.println("Panel Size= "+size());
		
		// If the size hasn't been set or if the windows was resized, resize this widget
		if ((d == null) || (d != size()) ) {
			d = size();
			if (alignment == VERTICAL)
				resize(new Dimension(2*border, d.height));
			else
				resize(new Dimension(d.width, 2*border));
		}
		int width = d.width-1, h = d.height-1;
		if (debug)
			System.out.println("Panel Size= "+size());
		
		// If the highlight and shadow colors were not specified,
		// make them lighter and darker versions of background color
		if (highlight == null)
			highlight = getBackground().brighter();

		if (shadow == null)
			shadow = getBackground().darker();
		
		// set the topLeft and bottomRight colors according to 
		// whether or the perspective is RAISED or SUNKEN
		Color topLeft = (perspective==RAISED)?highlight:shadow;
		Color bottomRight = (perspective==RAISED)?shadow:highlight;
			
		// Now draw the highlights and shadows
		g.setColor(topLeft);
		for(int i=0; i<border; i++) {
			g.drawLine(i,i,width-i,i);
			g.drawLine(i,i,i,h-i);
		}
		
		g.setColor(bottomRight);
		for(int i=0; i<border; i++) {
			g.drawLine(width-i,h-i, width-i,i);
			g.drawLine(width-i,h-i, i,h-i);
		}

		// Let the super do its regular paint stuff
		super.paint(g); 
	}
}
 

