/**
  NAME:         Thermometer.java
                Copyright (c) 1995 Liam Relihan. All rights reserved.
  PURPOSE:      
  AUTHOR:       Liam Relihan
                <relihanl@ul.ie>
  SCCS:         %M%        %R%.%L% %D%
  DATE:         15-May-1996
  PARAMETERS:   
  CALLED BY:
  MODIFIED:     
  BUGS:         
 */
package vt.cs.widgets;

import java.awt.*;
import java.lang.*;
import java.util.*;

public class Thermometer extends Meter {

   /**  Constructor for Thermometer class.
    * @arg lo     lowest value the thermometer can indicate
    * @arg hi     highest value the thermometer can indicate
    * @arg grad   graduations in which scale is marked
    * @arg val    initial value
    */
   public Thermometer(double lo,double hi,double grad,double val){
     super(lo,hi,grad,val);
   }


/** Draw the Thermometer with mercury */
  protected void drawMeter(Graphics g) {
    Color forecolor;
    int width=bounds().width;
    int height=bounds().height;
    int meterBorder=width/50;
    int t_base=(int)((double)height*0.7);
    int caption_y=(int)((double)height*0.85);
    int t_size=t_base-height/10;
    int t_width=t_size/10;
    int bulbDiameter=(int)((double)t_width*1.5);
    int bulbRadius=bulbDiameter/2;
    int t_xpos=(width/2)-(t_width/2);

    g.setColor(panelColor);
    
    g.fill3DRect(meterBorder,
		 meterBorder,
		 width-meterBorder*2,
		 height-meterBorder*2,
		 true);

    g.setColor(Color.white); 
    g.fillRoundRect(t_xpos,
		    t_base-t_size-t_width,
		    t_width,
		    t_size+t_width*2,
		    t_width,
		    t_width);

    g.fillOval(t_xpos+(t_width/2)-bulbRadius,
	       t_base,
	       bulbDiameter,
	       bulbDiameter);

    g.setColor(Color.black); 

    double sizeGrad=((double)t_size/(highestValue-lowestValue));

    g.setFont(numbersFont);
 
    for(double i=lowestValue;i<=highestValue;i=i+scaleGrad){

      int mark=t_base+(int)(lowestValue-(i*sizeGrad));
      g.drawLine(t_xpos+t_width+3, mark, t_xpos+t_width+8, mark);

      g.drawString(String.valueOf(i), t_xpos+t_width+10, mark); 

    }

    g.setColor(Color.red); 
    g.drawLine (t_xpos+(t_width/2),
		t_base+5,
		t_xpos+(t_width/2),
		t_base+(int)(lowestValue-(value*sizeGrad)));

     // draw caption
       g.setColor(Color.black); 
     g.setFont(captionFont);
     int caption_length=g.getFontMetrics().stringWidth(caption);
     g.drawString(caption,(width/2)-(caption_length/2),caption_y);

  }

   public Dimension minimumSize() { return new Dimension(250, 150); }
   
   public Dimension preferredSize() { return minimumSize(); }

}





